# -*- coding: utf-8 -*-
"""
Relationship Analysis Panel
===========================
Analyze relationships between TWO DIFFERENT bibliometric entity types.
Examples: Authors vs Keywords, Countries vs Sources, Keywords vs Years, etc.
Includes comprehensive item filtering (include/exclude lists, regex, file loading).
"""

import tkinter as tk
from tkinter import ttk, messagebox, filedialog
import threading
from typing import Dict, Optional, List, Tuple, Set
import io
import os
import re

from biblium.gui.config import FONTS, LAYOUT, get_theme
from biblium.gui.core.events import event_bus, EventBus
from biblium.gui.panels.base import BasePanel
from biblium.gui.widgets.cards import Card
from biblium.gui.widgets.buttons import ThemedButton, ActionButton
from biblium.gui.widgets.forms import LabeledCombobox, LabeledSpinbox, LabeledCheckbox, LabeledEntry

try:
    import pandas as pd
    import numpy as np
    HAS_PANDAS = True
except ImportError:
    HAS_PANDAS = False

try:
    import matplotlib
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    from PIL import Image, ImageTk
    HAS_MATPLOTLIB = True
except ImportError:
    HAS_MATPLOTLIB = False


class EntityFilterFrame(tk.Frame):
    """Reusable filtering widget for an entity - matches the design in the screenshot."""
    
    def __init__(self, parent, theme_dict, theme_name="light", **kwargs):
        self.theme = theme_dict
        self.theme_name = theme_name
        super().__init__(parent, bg=theme_dict["bg_card"], **kwargs)
        
        # Include only (one per line)
        inc_header = tk.Frame(self, bg=self.theme["bg_card"])
        inc_header.pack(fill=tk.X, pady=(0, 2))
        
        tk.Label(inc_header, text="Include only (one per line):", 
                 font=FONTS.get_font("small"), bg=self.theme["bg_card"], 
                 fg=self.theme["text_secondary"], anchor='w').pack(side=tk.LEFT)
        
        self.inc_load_btn = tk.Button(inc_header, text="📁 Load", font=FONTS.get_font("small"),
                                       relief=tk.FLAT, cursor="hand2",
                                       command=lambda: self._load_file("include"))
        self.inc_load_btn.pack(side=tk.RIGHT)
        
        self.include_text = tk.Text(self, height=4, width=30, font=FONTS.get_font("small"),
                                     relief=tk.SOLID, borderwidth=1)
        self.include_text.pack(fill=tk.X, pady=(0, 8))
        
        # Exclude (one per line)
        exc_header = tk.Frame(self, bg=self.theme["bg_card"])
        exc_header.pack(fill=tk.X, pady=(0, 2))
        
        tk.Label(exc_header, text="Exclude (one per line):", 
                 font=FONTS.get_font("small"), bg=self.theme["bg_card"], 
                 fg=self.theme["text_secondary"], anchor='w').pack(side=tk.LEFT)
        
        self.exc_load_btn = tk.Button(exc_header, text="📁 Load", font=FONTS.get_font("small"),
                                       relief=tk.FLAT, cursor="hand2",
                                       command=lambda: self._load_file("exclude"))
        self.exc_load_btn.pack(side=tk.RIGHT)
        
        self.exclude_text = tk.Text(self, height=4, width=30, font=FONTS.get_font("small"),
                                     relief=tk.SOLID, borderwidth=1)
        self.exclude_text.pack(fill=tk.X, pady=(0, 8))
        
        # Regex include
        regex_inc_frame = tk.Frame(self, bg=self.theme["bg_card"])
        regex_inc_frame.pack(fill=tk.X, pady=(0, 4))
        
        tk.Label(regex_inc_frame, text="Regex include:", 
                 font=FONTS.get_font("small"), bg=self.theme["bg_card"], 
                 fg=self.theme["text_secondary"], width=12, anchor='w').pack(side=tk.LEFT)
        
        self.regex_include = tk.Entry(regex_inc_frame, font=FONTS.get_font("small"),
                                       relief=tk.SOLID, borderwidth=1)
        self.regex_include.pack(side=tk.LEFT, fill=tk.X, expand=True)
        self.regex_include.insert(0, "")
        # Add placeholder
        self._add_placeholder(self.regex_include, "e.g., ^Smith|^Jones (regex pattern)")
        
        # Regex exclude
        regex_exc_frame = tk.Frame(self, bg=self.theme["bg_card"])
        regex_exc_frame.pack(fill=tk.X, pady=(0, 4))
        
        tk.Label(regex_exc_frame, text="Regex exclude:", 
                 font=FONTS.get_font("small"), bg=self.theme["bg_card"], 
                 fg=self.theme["text_secondary"], width=12, anchor='w').pack(side=tk.LEFT)
        
        self.regex_exclude = tk.Entry(regex_exc_frame, font=FONTS.get_font("small"),
                                       relief=tk.SOLID, borderwidth=1)
        self.regex_exclude.pack(side=tk.LEFT, fill=tk.X, expand=True)
        self.regex_exclude.insert(0, "")
        self._add_placeholder(self.regex_exclude, "e.g., Unknown|Anonymous")
        
        # Max items
        max_frame = tk.Frame(self, bg=self.theme["bg_card"])
        max_frame.pack(fill=tk.X, pady=(0, 4))
        
        tk.Label(max_frame, text="Max items:", 
                 font=FONTS.get_font("small"), bg=self.theme["bg_card"], 
                 fg=self.theme["text_secondary"], width=12, anchor='w').pack(side=tk.LEFT)
        
        self.max_items = tk.Spinbox(max_frame, from_=0, to=500, width=10, 
                                     font=FONTS.get_font("small"),
                                     relief=tk.SOLID, borderwidth=1)
        self.max_items.delete(0, tk.END)
        self.max_items.insert(0, "0")
        self.max_items.pack(side=tk.LEFT)
        
        # Skip header checkbox
        self.skip_header_var = tk.BooleanVar(value=True)
        self.skip_header_cb = tk.Checkbutton(self, text="Skip header row when loading files",
                                              variable=self.skip_header_var, 
                                              font=FONTS.get_font("small"),
                                              bg=self.theme["bg_card"], 
                                              fg=self.theme["text_secondary"],
                                              activebackground=self.theme["bg_card"],
                                              selectcolor=self.theme["bg_card"])
        self.skip_header_cb.pack(anchor='w', pady=(4, 4))
        
        # Note
        note_text = "Note: 'Include only' overrides Top N. Max items=0 means no limit.\nLoad from .txt, .csv, or .xlsx files."
        tk.Label(self, text=note_text, font=FONTS.get_font("small"), 
                 bg=self.theme["bg_card"], fg=self.theme["text_muted"], 
                 justify='left', anchor='w').pack(anchor='w')
    
    def _add_placeholder(self, entry, placeholder_text):
        """Add placeholder text to an entry widget."""
        entry.placeholder = placeholder_text
        entry.showing_placeholder = True
        entry.config(fg='gray')
        entry.delete(0, tk.END)
        entry.insert(0, placeholder_text)
        
        def on_focus_in(event):
            if entry.showing_placeholder:
                entry.delete(0, tk.END)
                entry.config(fg='black')
                entry.showing_placeholder = False
        
        def on_focus_out(event):
            if not entry.get():
                entry.showing_placeholder = True
                entry.config(fg='gray')
                entry.insert(0, entry.placeholder)
        
        entry.bind("<FocusIn>", on_focus_in)
        entry.bind("<FocusOut>", on_focus_out)
    
    def _load_file(self, target):
        """Load items from file into include or exclude text area."""
        filepath = filedialog.askopenfilename(
            title=f"Select file for {target}",
            filetypes=[("Text files", "*.txt"), ("CSV files", "*.csv"), 
                      ("Excel files", "*.xlsx"), ("All files", "*.*")]
        )
        if not filepath:
            return
        
        try:
            items = []
            skip_header = self.skip_header_var.get()
            
            if filepath.endswith('.xlsx'):
                try:
                    df = pd.read_excel(filepath, header=0 if skip_header else None)
                    items = df.iloc[:, 0].dropna().astype(str).tolist()
                except Exception as e:
                    messagebox.showerror("Error", f"Failed to read Excel file: {e}")
                    return
            elif filepath.endswith('.csv'):
                try:
                    df = pd.read_csv(filepath, header=0 if skip_header else None)
                    items = df.iloc[:, 0].dropna().astype(str).tolist()
                except Exception as e:
                    messagebox.showerror("Error", f"Failed to read CSV file: {e}")
                    return
            else:
                # Text file
                with open(filepath, 'r', encoding='utf-8') as f:
                    lines = f.readlines()
                    start_idx = 1 if skip_header and lines else 0
                    for line in lines[start_idx:]:
                        line = line.strip()
                        if line and not line.startswith('#'):
                            items.append(line)
            
            # Insert into appropriate text area
            text_widget = self.include_text if target == "include" else self.exclude_text
            text_widget.delete("1.0", tk.END)
            text_widget.insert("1.0", "\n".join(items))
            
            messagebox.showinfo("Loaded", f"Loaded {len(items)} items from file.")
            
        except Exception as e:
            messagebox.showerror("Error", f"Failed to load file: {e}")
    
    def _get_entry_value(self, entry):
        """Get entry value, ignoring placeholder text."""
        if hasattr(entry, 'showing_placeholder') and entry.showing_placeholder:
            return ""
        return entry.get().strip()
    
    def get_filter_config(self) -> dict:
        """Get the current filter configuration."""
        include_items = set()
        exclude_items = set()
        
        # Parse include text
        inc_text = self.include_text.get("1.0", tk.END).strip()
        if inc_text:
            for line in inc_text.split('\n'):
                line = line.strip()
                if line:
                    include_items.add(line)
        
        # Parse exclude text
        exc_text = self.exclude_text.get("1.0", tk.END).strip()
        if exc_text:
            for line in exc_text.split('\n'):
                line = line.strip()
                if line:
                    exclude_items.add(line)
        
        # Get regex patterns (check for placeholder)
        regex_inc = self._get_entry_value(self.regex_include)
        regex_exc = self._get_entry_value(self.regex_exclude)
        
        # Get max items
        try:
            max_items = int(self.max_items.get())
        except ValueError:
            max_items = 0
        
        return {
            "include_items": include_items,
            "exclude_items": exclude_items,
            "regex_include": regex_inc,
            "regex_exclude": regex_exc,
            "max_items": max_items,
        }


class RelationshipPanel(BasePanel):
    """Panel for analyzing relationships between two different entity types."""
    
    title = "Relationship Analysis"
    icon = "🔗"
    description = "Analyze relationships between two entity types"
    requires_data = True
    
    # Entity configurations: (column_candidates, value_type, separator)
    ENTITY_CONFIG = {
        "Author Keywords": (["Processed Author Keywords", "Author Keywords", "DE"], "list", "; "),
        "Index Keywords": (["Processed Index Keywords", "Index Keywords", "ID"], "list", "; "),
        "Authors": (["Authors", "Authors or Inventors", "AU"], "list", "; "),
        "Sources": (["Source title", "Source Title", "Journal", "SO"], "string", None),
        "Countries": (["Countries", "Country"], "list", "; "),
        "Affiliations": (["Affiliations", "Addresses", "C1"], "list", "; "),
        "Year": (["Year", "Publication Year", "PY"], "string", None),
        "Document Type": (["Document Type", "DT"], "string", None),
        "Language": (["Language", "Language of Original Document", "LA"], "string", None),
        "Subject Area": (["Subject Area", "WC", "SC"], "list", "; "),
    }
    
    VIS_TYPES = ["Contingency Heatmap", "Cluster Heatmap", "Top Pairs (Bubble Chart)", 
                 "Correspondence Analysis", "Sankey Diagram"]
    
    ASSOC_MEASURES = ["Raw Counts", "Chi-square Residuals", "Normalized (Row %)", 
                     "Normalized (Column %)", "Log Odds Ratio"]
    
    def __init__(self, parent, theme: str = "light", bib=None, **kwargs):
        self._current_fig = None
        self._photo_image = None
        self._contingency_matrix = None
        self._last_image_bytes = None
        super().__init__(parent, theme=theme, bib=bib, **kwargs)
        self._primary_action = self._run_analysis
    
    def _create_options(self):
        """Create the options panel."""
        self._add_title()
        
        # Entity 1 Selection (Rows)
        entity1_card = Card(self.options_content, title="📊 Row Entity", theme=self.theme_name)
        entity1_card.pack(fill=tk.X, padx=8, pady=4)
        
        self.entity1_combo = LabeledCombobox(
            entity1_card.content, label="Entity:", values=list(self.ENTITY_CONFIG.keys()),
            default="Authors", theme=self.theme_name, label_width=8)
        self.entity1_combo.pack(fill=tk.X, pady=2)
        
        self.top_n1_spin = LabeledSpinbox(
            entity1_card.content, label="Top N:", from_=5, to=200, default=10,
            theme=self.theme_name, label_width=8)
        self.top_n1_spin.pack(fill=tk.X, pady=2)
        
        # Filtering for Entity 1
        filter1_card = Card(self.options_content, title="🔍 Row Entity Filtering", theme=self.theme_name)
        filter1_card.pack(fill=tk.X, padx=8, pady=4)
        
        self.filter1 = EntityFilterFrame(filter1_card.content, self.theme, self.theme_name)
        self.filter1.pack(fill=tk.X)
        
        # Entity 2 Selection (Columns)
        entity2_card = Card(self.options_content, title="📋 Column Entity", theme=self.theme_name)
        entity2_card.pack(fill=tk.X, padx=8, pady=4)
        
        self.entity2_combo = LabeledCombobox(
            entity2_card.content, label="Entity:", values=list(self.ENTITY_CONFIG.keys()),
            default="Author Keywords", theme=self.theme_name, label_width=8)
        self.entity2_combo.pack(fill=tk.X, pady=2)
        
        self.top_n2_spin = LabeledSpinbox(
            entity2_card.content, label="Top N:", from_=5, to=200, default=10,
            theme=self.theme_name, label_width=8)
        self.top_n2_spin.pack(fill=tk.X, pady=2)
        
        # Filtering for Entity 2
        filter2_card = Card(self.options_content, title="🔍 Column Entity Filtering", theme=self.theme_name)
        filter2_card.pack(fill=tk.X, padx=8, pady=4)
        
        self.filter2 = EntityFilterFrame(filter2_card.content, self.theme, self.theme_name)
        self.filter2.pack(fill=tk.X)
        
        # Visualization Settings
        vis_card = Card(self.options_content, title="📈 Visualization", theme=self.theme_name)
        vis_card.pack(fill=tk.X, padx=8, pady=4)
        
        self.vis_combo = LabeledCombobox(
            vis_card.content, label="Plot:", values=self.VIS_TYPES,
            default="Contingency Heatmap", theme=self.theme_name, label_width=8)
        self.vis_combo.pack(fill=tk.X, pady=2)
        self.vis_combo.bind("<<ComboboxSelected>>", self._on_vis_change)
        
        self.measure_combo = LabeledCombobox(
            vis_card.content, label="Measure:", values=self.ASSOC_MEASURES,
            default="Raw Counts", theme=self.theme_name, label_width=8)
        self.measure_combo.pack(fill=tk.X, pady=2)
        
        # Top Pairs Settings (shown conditionally)
        self.pairs_frame = tk.Frame(vis_card.content, bg=self.theme["bg_card"])
        
        self.pairs_top_n = LabeledSpinbox(
            self.pairs_frame, label="Top pairs:", from_=10, to=100, default=30,
            theme=self.theme_name, label_width=8)
        self.pairs_top_n.pack(fill=tk.X, pady=2)
        
        self.pairs_sign = LabeledCombobox(
            self.pairs_frame, label="Show:", values=["Both (+/-)", "Positive only", "Negative only"],
            default="Both (+/-)", theme=self.theme_name, label_width=8)
        self.pairs_sign.pack(fill=tk.X, pady=2)
        
        # Appearance Settings
        appear_card = Card(self.options_content, title="🎨 Appearance", theme=self.theme_name)
        appear_card.pack(fill=tk.X, padx=8, pady=4)
        
        self.colormap_combo = LabeledCombobox(
            appear_card.content, label="Colormap:",
            values=["viridis", "plasma", "coolwarm", "RdBu_r", "YlGnBu", "Blues", "Reds", "Spectral", "YlOrRd"],
            default="YlGnBu", theme=self.theme_name, label_width=8)
        self.colormap_combo.pack(fill=tk.X, pady=2)
        
        self.show_values_cb = LabeledCheckbox(
            appear_card.content, label="Show values in cells", default=True, theme=self.theme_name)
        self.show_values_cb.pack(fill=tk.X, pady=2)
        
        # Run Button
        # LLM Configuration (v2.11)
        self._create_llm_config_card()
        
        btn_frame = tk.Frame(self.options_content, bg=self.theme["bg_secondary"])
        btn_frame.pack(fill=tk.X, padx=8, pady=8)
        
        ActionButton(btn_frame, text="Run Analysis", icon="🔗",
            command=self._run_analysis, theme=self.theme_name).pack(fill=tk.X)
        
        ThemedButton(btn_frame, text="Export Results", style="secondary",
            command=self._export_results, theme=self.theme_name).pack(fill=tk.X, pady=(4, 0))
    
    def _on_vis_change(self, event=None):
        """Update UI based on visualization type."""
        vis_type = self.vis_combo.get()
        if vis_type == "Top Pairs (Bubble Chart)":
            self.pairs_frame.pack(fill=tk.X, pady=2)
            self.colormap_combo.set("coolwarm")
            self.measure_combo.set("Chi-square Residuals")
        else:
            self.pairs_frame.pack_forget()
            if vis_type in ["Contingency Heatmap", "Cluster Heatmap"]:
                self.colormap_combo.set("YlGnBu")
    
    def _create_results(self):
        """Create results panel - optimized for larger plot display."""
        self.results_card = tk.Frame(
            self.results_frame, bg=self.theme["bg_card"],
            highlightbackground=self.theme["border"], highlightthickness=1)
        self.results_card.pack(fill=tk.BOTH, expand=True, padx=2, pady=2)
        
        self.results_notebook = ttk.Notebook(self.results_card)
        self.results_notebook.pack(fill=tk.BOTH, expand=True)
        
        self.plot_frame = tk.Frame(self.results_notebook, bg=self.theme["bg_card"])
        self.results_notebook.add(self.plot_frame, text="📊 Plot")
        
        self.image_label = tk.Label(self.plot_frame, bg=self.theme["bg_card"])
        self.image_label.pack(fill=tk.BOTH, expand=True)
        
        self.data_frame = tk.Frame(self.results_notebook, bg=self.theme["bg_card"])
        self.results_notebook.add(self.data_frame, text="📋 Table")
        
        # Info tab
        info_frame = tk.Frame(self.results_notebook, bg=self.theme["bg_card"])
        self.results_notebook.add(info_frame, text="ℹ️ Info")
        self._create_info_content(info_frame)
        
        self.tree_frame = tk.Frame(self.data_frame, bg=self.theme["bg_card"])
        self.tree_frame.pack(fill=tk.BOTH, expand=True, padx=2, pady=2)
        
        # Note: ScaledImageFrame handles its own resize events
    
    def _on_plot_resize(self, event=None):
        # Deprecated - ScaledImageFrame handles resize internally
        pass
    
    def _reset_resize_flag(self):
        pass
    
    def _show_placeholder(self, message: str):
        """Show a placeholder message in the plot frame."""
        # Clear all widgets in plot_frame
        for widget in self.plot_frame.winfo_children():
            try:
                widget.destroy()
            except tk.TclError:
                pass
        
        # Create a new label for the message
        self.image_label = tk.Label(
            self.plot_frame, 
            text=message, 
            font=FONTS.get_font("body"), 
            bg=self.theme["bg_card"],
            fg=self.theme["text_muted"]
        )
        self.image_label.pack(expand=True)
    
    def _apply_filters(self, items: List[str], filter_config: dict, top_n: int) -> List[str]:
        """Apply all filters to a list of items."""
        result = items.copy()
        
        # Apply include filter (if specified, only keep matching items)
        if filter_config["include_items"]:
            result = [i for i in result if i in filter_config["include_items"]]
        
        # Apply exclude filter
        if filter_config["exclude_items"]:
            result = [i for i in result if i not in filter_config["exclude_items"]]
        
        # Apply regex include
        if filter_config["regex_include"]:
            try:
                pattern = re.compile(filter_config["regex_include"], re.IGNORECASE)
                result = [i for i in result if pattern.search(i)]
            except re.error:
                pass
        
        # Apply regex exclude
        if filter_config["regex_exclude"]:
            try:
                pattern = re.compile(filter_config["regex_exclude"], re.IGNORECASE)
                result = [i for i in result if not pattern.search(i)]
            except re.error:
                pass
        
        # Apply max items limit
        max_items = filter_config["max_items"]
        if max_items > 0:
            result = result[:max_items]
        elif not filter_config["include_items"]:
            # If no include filter and max_items=0, use top_n
            result = result[:top_n]
        
        return result
    
    def _run_analysis(self):
        """Run relationship analysis between two entity types."""
        if not self.bib:
            messagebox.showwarning("No Data", "Please load a dataset first.")
            return
        
        entity1 = self.entity1_combo.get()
        entity2 = self.entity2_combo.get()
        
        if entity1 == entity2:
            messagebox.showwarning("Same Entity", 
                "Please select two different entity types.\nFor co-occurrence within the same entity, use Co-occurrence Networks.")
            return
        
        event_bus.emit(EventBus.ANALYSIS_STARTED, {"name": self.title})
        self._is_running = True
        self._show_placeholder(f"⏳ Computing: {entity1} × {entity2}...")
        
        top_n1, top_n2 = self.top_n1_spin.get(), self.top_n2_spin.get()
        vis_type, measure = self.vis_combo.get(), self.measure_combo.get()
        colormap, show_values = self.colormap_combo.get(), self.show_values_cb.get()
        
        # Get filter configurations
        filter1_config = self.filter1.get_filter_config()
        filter2_config = self.filter2.get_filter_config()
        
        def do_analysis():
            try:
                config1, config2 = self.ENTITY_CONFIG.get(entity1), self.ENTITY_CONFIG.get(entity2)
                if not config1 or not config2:
                    raise ValueError("Unknown entity configuration")
                
                col1_candidates, value_type1, sep1 = config1
                col2_candidates, value_type2, sep2 = config2
                
                col1 = self._find_column(col1_candidates)
                col2 = self._find_column(col2_candidates)
                
                if not col1:
                    raise ValueError(f"Column not found for {entity1}")
                if not col2:
                    raise ValueError(f"Column not found for {entity2}")
                
                print(f"[DEBUG] Relationship: {entity1} ({col1}) × {entity2} ({col2})")
                
                contingency = self._build_contingency_matrix(
                    col1, col2, value_type1, value_type2, sep1, sep2, 
                    top_n1, top_n2, entity1, entity2,
                    filter1_config, filter2_config)
                
                if contingency is None or contingency.empty:
                    raise ValueError("No relationship data found")
                
                self._contingency_matrix = contingency
                display_matrix = self._compute_measure(contingency, measure)
                
                result = self._create_visualization(display_matrix, contingency, vis_type, 
                    entity1, entity2, colormap, show_values, measure)
                
                if result is not None:
                    # Check if result is bytes (from Sankey) or a matplotlib figure
                    if isinstance(result, bytes):
                        image_bytes = result
                        self._current_fig = None
                    else:
                        # It's a matplotlib figure
                        fig = result
                        self._current_fig = fig
                        buf = io.BytesIO()
                        fig.savefig(buf, format='png', dpi=150, bbox_inches='tight', facecolor='white')
                        buf.seek(0)
                        image_bytes = buf.getvalue()
                        buf.close()
                        plt.close(fig)
                    self.after(0, lambda ib=image_bytes: self._show_results(ib, display_matrix))
                else:
                    self.after(0, lambda: self._on_error("Failed to create visualization"))
                    
            except Exception as e:
                import traceback
                traceback.print_exc()
                error_msg = str(e)
                self.after(0, lambda msg=error_msg: self._on_error(msg))
        
        threading.Thread(target=do_analysis, daemon=True).start()
    
    def _find_column(self, candidates):
        if not self.bib or not hasattr(self.bib, 'df'):
            return None
        for col in candidates:
            if col in self.bib.df.columns:
                return col
            for df_col in self.bib.df.columns:
                if df_col.lower() == col.lower():
                    return df_col
        return None
    
    def _build_contingency_matrix(self, col1, col2, vt1, vt2, sep1, sep2, 
                                   top_n1, top_n2, name1, name2,
                                   filter1_config, filter2_config):
        df = self.bib.df.copy()
        default_sep = getattr(self.bib, 'default_separator', '; ')
        sep1, sep2 = sep1 or default_sep, sep2 or default_sep
        
        def extract_items(series, vt, sep):
            if vt == "list":
                items = series.dropna().astype(str).str.split(sep).explode().str.strip()
            else:
                items = series.dropna().astype(str).str.strip()
            return items[items != '']
        
        items1, items2 = extract_items(df[col1], vt1, sep1), extract_items(df[col2], vt2, sep2)
        
        # Get candidate items (more than needed for filtering)
        has_filter1 = filter1_config["include_items"] or filter1_config["exclude_items"] or filter1_config["regex_include"] or filter1_config["regex_exclude"]
        has_filter2 = filter2_config["include_items"] or filter2_config["exclude_items"] or filter2_config["regex_include"] or filter2_config["regex_exclude"]
        
        multiplier1 = 5 if has_filter1 else 1
        multiplier2 = 5 if has_filter2 else 1
        
        candidates1 = items1.value_counts().head(top_n1 * multiplier1).index.tolist()
        candidates2 = items2.value_counts().head(top_n2 * multiplier2).index.tolist()
        
        # Apply filters
        top1 = self._apply_filters(candidates1, filter1_config, top_n1)
        top2 = self._apply_filters(candidates2, filter2_config, top_n2)
        
        if not top1 or not top2:
            return pd.DataFrame()
        
        print(f"[DEBUG] After filtering: {len(top1)} × {len(top2)} items")
        
        contingency = pd.DataFrame(0, index=top1, columns=top2)
        contingency.index.name, contingency.columns.name = name1, name2
        
        for _, row in df.iterrows():
            v1, v2 = row[col1], row[col2]
            if pd.isna(v1) or pd.isna(v2):
                continue
            
            di1 = [x.strip() for x in str(v1).split(sep1) if x.strip()] if vt1 == "list" else [str(v1).strip()]
            di2 = [x.strip() for x in str(v2).split(sep2) if x.strip()] if vt2 == "list" else [str(v2).strip()]
            
            for i1 in di1:
                if i1 in top1:
                    for i2 in di2:
                        if i2 in top2:
                            contingency.loc[i1, i2] += 1
        
        return contingency.loc[(contingency.sum(axis=1) > 0), (contingency.sum(axis=0) > 0)]
    
    def _compute_measure(self, cont, measure):
        if measure == "Raw Counts":
            return cont
        
        obs = cont.values.astype(float)
        row_s, col_s, total = obs.sum(axis=1, keepdims=True), obs.sum(axis=0, keepdims=True), obs.sum()
        
        if total == 0:
            return cont
        
        if measure == "Chi-square Residuals":
            exp = (row_s @ col_s) / total
            with np.errstate(divide='ignore', invalid='ignore'):
                res = (obs - exp) / np.sqrt(exp)
                res = np.nan_to_num(res, nan=0.0, posinf=0.0, neginf=0.0)
            return pd.DataFrame(res, index=cont.index, columns=cont.columns)
        
        elif measure == "Normalized (Row %)":
            return cont.div(cont.sum(axis=1), axis=0).fillna(0) * 100
        
        elif measure == "Normalized (Column %)":
            return cont.div(cont.sum(axis=0), axis=1).fillna(0) * 100
        
        elif measure == "Log Odds Ratio":
            obs_s = obs + 0.5
            exp = (obs_s.sum(axis=1, keepdims=True) @ obs_s.sum(axis=0, keepdims=True)) / obs_s.sum()
            with np.errstate(divide='ignore', invalid='ignore'):
                lo = np.log(obs_s / exp)
                lo = np.nan_to_num(lo, nan=0.0, posinf=3.0, neginf=-3.0)
            return pd.DataFrame(lo, index=cont.index, columns=cont.columns)
        
        return cont
    
    def _create_visualization(self, disp, raw, vis, e1, e2, cmap, show_val, measure):
        plt.ioff()
        if vis == "Contingency Heatmap":
            return self._plot_heatmap(disp, e1, e2, cmap, show_val, measure)
        elif vis == "Cluster Heatmap":
            return self._plot_clustermap(disp, e1, e2, cmap, show_val, measure)
        elif vis == "Top Pairs (Bubble Chart)":
            return self._plot_top_pairs(disp, e1, e2, cmap, measure)
        elif vis == "Correspondence Analysis":
            return self._plot_correspondence(raw, e1, e2)
        elif vis == "Sankey Diagram":
            return self._plot_sankey(raw, e1, e2, cmap)
        return None
    
    def _plot_heatmap(self, mat, e1, e2, cmap, show_val, measure):
        import seaborn as sns
        fig, ax = plt.subplots(figsize=(12, 10))
        
        fmt = ".0f" if measure == "Raw Counts" else ".1f" if "%" in measure else ".2f"
        center = 0 if measure in ["Chi-square Residuals", "Log Odds Ratio"] else None
        if center is not None:
            cmap = "coolwarm"
        
        sns.heatmap(mat, annot=show_val, fmt=fmt, cmap=cmap, center=center,
            cbar_kws={"label": measure}, ax=ax, annot_kws={"fontsize": 8})
        
        ax.set_title(f"{e1} × {e2}", fontsize=14, pad=10)
        ax.set_xlabel(e2, fontsize=11)
        ax.set_ylabel(e1, fontsize=11)
        plt.xticks(rotation=45, ha='right')
        plt.yticks(rotation=0)
        fig.tight_layout()
        return fig
    
    def _plot_clustermap(self, mat, e1, e2, cmap, show_val, measure):
        import seaborn as sns
        if mat.shape[0] < 2 or mat.shape[1] < 2:
            return self._plot_heatmap(mat, e1, e2, cmap, show_val, measure)
        
        fmt = ".0f" if measure == "Raw Counts" else ".1f" if "%" in measure else ".2f"
        center = 0 if measure in ["Chi-square Residuals", "Log Odds Ratio"] else None
        if center is not None:
            cmap = "coolwarm"
        
        g = sns.clustermap(mat, annot=show_val, fmt=fmt, cmap=cmap, center=center,
            figsize=(12, 10), dendrogram_ratio=0.12, cbar_kws={"label": measure}, annot_kws={"fontsize": 7})
        
        g.fig.suptitle(f"Clustered: {e1} × {e2}", y=1.01, fontsize=14)
        plt.setp(g.ax_heatmap.get_xticklabels(), rotation=45, ha='right', fontsize=9)
        plt.setp(g.ax_heatmap.get_yticklabels(), rotation=0, fontsize=9)
        g.ax_heatmap.set_xlabel(e2, fontsize=11)
        g.ax_heatmap.set_ylabel(e1, fontsize=11)
        return g.fig
    
    def _plot_top_pairs(self, mat, e1, e2, cmap, measure):
        top_n = self.pairs_top_n.get()
        sign_opt = self.pairs_sign.get()
        
        pairs = []
        for r in mat.index:
            for c in mat.columns:
                v = mat.loc[r, c]
                if pd.notna(v) and v != 0:
                    raw = self._contingency_matrix.loc[r, c] if self._contingency_matrix is not None else abs(v)
                    pairs.append({"Row": str(r), "Col": str(c), "Val": float(v), "Cnt": float(raw)})
        
        if not pairs:
            fig, ax = plt.subplots(figsize=(10, 8))
            ax.text(0.5, 0.5, "No pairs found", ha='center', va='center')
            ax.set_axis_off()
            return fig
        
        pdf = pd.DataFrame(pairs)
        if sign_opt == "Positive only":
            pdf = pdf[pdf["Val"] > 0]
        elif sign_opt == "Negative only":
            pdf = pdf[pdf["Val"] < 0]
        
        pdf["abs"] = pdf["Val"].abs()
        pdf = pdf.sort_values("abs", ascending=False).head(top_n)
        
        if pdf.empty:
            fig, ax = plt.subplots(figsize=(10, 8))
            ax.text(0.5, 0.5, "No pairs match criteria", ha='center', va='center')
            ax.set_axis_off()
            return fig
        
        fig, ax = plt.subplots(figsize=(14, 10))
        
        r_lbl = list(pdf.groupby("Row")["abs"].sum().sort_values(ascending=False).index)
        c_lbl = list(pdf.groupby("Col")["abs"].sum().sort_values(ascending=False).index)
        
        x_pos, y_pos = {l: i for i, l in enumerate(r_lbl)}, {l: i for i, l in enumerate(c_lbl)}
        pdf = pdf[pdf["Row"].isin(r_lbl) & pdf["Col"].isin(c_lbl)]
        
        x = [x_pos[r] for r in pdf["Row"]]
        y = [y_pos[c] for c in pdf["Col"]]
        max_cnt = pdf["Cnt"].max() if pdf["Cnt"].max() > 0 else 1
        sizes = 50 + (pdf["Cnt"] / max_cnt) * 500
        
        sc = ax.scatter(x, y, s=sizes, c=pdf["Val"], cmap=cmap, alpha=0.7, edgecolors='white', linewidth=1)
        plt.colorbar(sc, ax=ax, label=measure)
        
        ax.set_xticks(range(len(r_lbl)))
        ax.set_xticklabels(r_lbl, rotation=45, ha='right', fontsize=9)
        ax.set_yticks(range(len(c_lbl)))
        ax.set_yticklabels(c_lbl, fontsize=9)
        ax.set_xlabel(e1, fontsize=11)
        ax.set_ylabel(e2, fontsize=11)
        ax.set_title(f"Top {len(pdf)} Pairs: {e1} × {e2}", fontsize=14, pad=10)
        ax.grid(True, alpha=0.3)
        fig.tight_layout()
        return fig
    
    def _plot_correspondence(self, cont, e1, e2):
        from sklearn.decomposition import TruncatedSVD
        
        data = np.nan_to_num(cont.values.astype(float), nan=0.0)
        row_s, col_s, total = data.sum(axis=1, keepdims=True), data.sum(axis=0, keepdims=True), data.sum()
        
        if total == 0 or min(data.shape) < 3:
            fig, ax = plt.subplots(figsize=(10, 8))
            ax.text(0.5, 0.5, "Not enough data for CA", ha='center', va='center')
            ax.set_axis_off()
            return fig
        
        exp = (row_s @ col_s) / total
        with np.errstate(divide='ignore', invalid='ignore'):
            res = (data - exp) / np.sqrt(exp)
            res = np.nan_to_num(res, nan=0.0, posinf=0.0, neginf=0.0)
        
        svd = TruncatedSVD(n_components=2)
        row_c = svd.fit_transform(res)
        col_c = svd.components_.T * svd.singular_values_
        expl = svd.explained_variance_ratio_ * 100
        
        fig, ax = plt.subplots(figsize=(12, 10))
        
        ax.scatter(row_c[:, 0], row_c[:, 1], c='steelblue', s=120, alpha=0.7, label=e1)
        for i, l in enumerate(cont.index):
            ax.annotate(str(l), (row_c[i, 0], row_c[i, 1]), fontsize=9, color='steelblue', fontweight='bold')
        
        ax.scatter(col_c[:, 0], col_c[:, 1], c='coral', s=120, alpha=0.7, marker='s', label=e2)
        for i, l in enumerate(cont.columns):
            ax.annotate(str(l), (col_c[i, 0], col_c[i, 1]), fontsize=9, color='coral', fontweight='bold')
        
        ax.axhline(0, color='gray', lw=0.5, ls='--', alpha=0.5)
        ax.axvline(0, color='gray', lw=0.5, ls='--', alpha=0.5)
        ax.set_xlabel(f"Dim 1 ({expl[0]:.1f}%)", fontsize=11)
        ax.set_ylabel(f"Dim 2 ({expl[1]:.1f}%)", fontsize=11)
        ax.set_title(f"Correspondence Analysis: {e1} × {e2}", fontsize=14, pad=10)
        ax.legend(loc='best')
        ax.grid(True, alpha=0.3)
        fig.tight_layout()
        return fig
    
    def _plot_sankey(self, cont, e1, e2, cmap):
        """Create Sankey diagram using plotbib implementation."""
        from biblium import plotbib
        import tempfile
        import os
        
        # Use plotbib's bipartite sankey function
        fig = plotbib.plot_bipartite_sankey(
            contingency_matrix=cont,
            left_label=e1,
            right_label=e2,
            title=f"Flow: {e1} → {e2}",
            max_links=30,
            min_value=0,
            colorscale="Blues",
            width=1000,
            height=700,
        )
        
        # Convert plotly figure to static image bytes
        try:
            img_bytes = fig.to_image(format="png", scale=2)
            return img_bytes
        except Exception as e:
            # Fallback: save to temp file and read
            try:
                with tempfile.NamedTemporaryFile(suffix='.png', delete=False) as tmp:
                    tmp_path = tmp.name
                fig.write_image(tmp_path, scale=2)
                with open(tmp_path, 'rb') as f:
                    img_bytes = f.read()
                os.unlink(tmp_path)
                return img_bytes
            except Exception as e2:
                print(f"Sankey export error: {e2}")
                # Return None to trigger error
                return None
    
    def _display_image(self, img_bytes):
        # Clear existing widgets in plot_frame
        for widget in self.plot_frame.winfo_children():
            widget.destroy()
        
        try:
            from biblium.gui.widgets.plots import ScaledImageFrame
            
            buf = io.BytesIO(img_bytes)
            img = Image.open(buf)
            
            scaled_frame = ScaledImageFrame(
                self.plot_frame, 
                theme=self.theme_name,
                maintain_aspect=True,
                max_scale=1.5
            )
            scaled_frame.pack(fill=tk.BOTH, expand=True)
            scaled_frame.set_image(img)
            
            self._scaled_frame = scaled_frame
        except Exception as e:
            # Fallback to original method
            buf = io.BytesIO(img_bytes)
            img = Image.open(buf)
            self._photo_image = ImageTk.PhotoImage(img)
            self.image_label = tk.Label(self.plot_frame, image=self._photo_image, bg=self.theme["bg_card"])
            self.image_label.pack(fill=tk.BOTH, expand=True)
    
    def _show_results(self, img_bytes, mat):
        self._is_running = False
        self._last_image_bytes = img_bytes
        event_bus.emit(EventBus.ANALYSIS_COMPLETED, {"name": self.title})
        
        self._display_image(img_bytes)
        self._update_matrix_table(mat)
    
    def _update_matrix_table(self, mat):
        for w in self.tree_frame.winfo_children():
            w.destroy()
        
        cols = [mat.index.name or "Row"] + [str(c) for c in mat.columns]
        tree = ttk.Treeview(self.tree_frame, columns=cols, show='headings', height=15)
        
        vsb = ttk.Scrollbar(self.tree_frame, orient="vertical", command=tree.yview)
        hsb = ttk.Scrollbar(self.tree_frame, orient="horizontal", command=tree.xview)
        tree.configure(yscrollcommand=vsb.set, xscrollcommand=hsb.set)
        
        tree.column(cols[0], width=120, anchor='w')
        tree.heading(cols[0], text=cols[0])
        for c in cols[1:]:
            tree.column(c, width=80, anchor='center')
            tree.heading(c, text=str(c)[:15])
        
        for idx, row in mat.iterrows():
            vals = [str(idx)[:20]] + [f"{v:.1f}" if isinstance(v, (int, float)) else str(v) for v in row]
            tree.insert("", "end", values=vals)
        
        tree.grid(row=0, column=0, sticky='nsew')
        vsb.grid(row=0, column=1, sticky='ns')
        hsb.grid(row=1, column=0, sticky='ew')
        self.tree_frame.grid_rowconfigure(0, weight=1)
        self.tree_frame.grid_columnconfigure(0, weight=1)
    
    def _on_error(self, msg):
        self._is_running = False
        event_bus.emit(EventBus.ERROR_OCCURRED, {"message": msg})
        self._show_placeholder(f"❌ Error: {msg}")
        messagebox.showerror("Error", msg)
    
    def _create_info_content(self, parent):
        """Create Info tab with documentation."""
        info_text = """
RELATIONSHIP ANALYSIS
═════════════════════

Analyze entity co-occurrence relationships.

RELATIONSHIP TYPES
──────────────────
• Author-Keyword
• Journal-Topic
• Country-Field
• Institution-Theme

METHODS
───────
• Co-occurrence matrix
• Association strength
• Correlation analysis
• Network analysis

OUTPUT
──────
• Relationship matrix
• Heatmap visualization
• Network diagram
• Top associations

METRICS
───────
• Co-occurrence count
• Jaccard similarity
• Lift (above expected)
• PMI (pointwise mutual info)
"""
        text_widget = tk.Text(
            parent, font=FONTS.get_font("body"),
            bg=self.theme["bg_card"], fg=self.theme["text_primary"],
            wrap=tk.WORD, padx=16, pady=16, relief=tk.FLAT,
        )
        text_widget.insert("1.0", info_text)
        text_widget.config(state=tk.DISABLED)
        text_widget.pack(fill=tk.BOTH, expand=True)
        
        # Add right-click context menu for copy
        def show_context_menu(event):
            menu = tk.Menu(text_widget, tearoff=0)
            menu.add_command(label="Copy Selected", command=lambda: copy_selected(text_widget))
            menu.add_command(label="Copy All", command=lambda: copy_all(text_widget))
            menu.tk_popup(event.x_root, event.y_root)
        
        def copy_selected(widget):
            try:
                widget.config(state=tk.NORMAL)
                selected = widget.get(tk.SEL_FIRST, tk.SEL_LAST)
                widget.config(state=tk.DISABLED)
                widget.clipboard_clear()
                widget.clipboard_append(selected)
            except tk.TclError:
                pass  # No selection
        
        def copy_all(widget):
            widget.config(state=tk.NORMAL)
            content = widget.get("1.0", tk.END)
            widget.config(state=tk.DISABLED)
            widget.clipboard_clear()
            widget.clipboard_append(content.strip())
        
        text_widget.bind("<Button-3>", show_context_menu)  # Right-click
        text_widget.bind("<Control-c>", lambda e: copy_selected(text_widget))

    def _export_results(self):
        if self._contingency_matrix is None:
            messagebox.showwarning("No Results", "Run analysis first.")
            return
        
        fp = filedialog.asksaveasfilename(defaultextension=".xlsx",
            filetypes=[("Excel", "*.xlsx"), ("CSV", "*.csv"), ("All", "*.*")])
        
        if fp:
            try:
                if fp.endswith('.xlsx'):
                    self._contingency_matrix.to_excel(fp, sheet_name='Contingency')
                else:
                    self._contingency_matrix.to_csv(fp)
                messagebox.showinfo("Exported", f"Saved to {fp}")
                if self._current_fig:
                    self._current_fig.savefig(fp.rsplit('.', 1)[0] + '_plot.png', dpi=300, bbox_inches='tight')
            except Exception as e:
                messagebox.showerror("Export Error", str(e))



# -*- coding: utf-8 -*-
"""
Biblium Base Module - Shared initialization and helper methods.

This module provides the common foundation for BiblioAnalysis and BiblioGroupAnalysis,
eliminating code duplication while maintaining backward compatibility.

@author: Lan.Umek
@refactored: Claude (Anthropic)
"""

import os
import pandas as pd
from typing import Callable, Optional, Dict, List, Union


# =============================================================================
# DATABASE SEPARATOR CONFIGURATION
# =============================================================================

SEPARATOR_MAP = {
    # Major databases
    "scopus": "; ",
    "open alex": "|",
    "oa": "|",
    "wos": "; ",
    "pubmed": "; ",
    "dimensions": "; ",
    "lens": "; ",
    # Engineering/CS
    "ieee": "; ",
    "dblp": "; ",
    "arxiv": "; ",
    "semantic scholar": "; ",
    "s2": "; ",
    # Social sciences
    "proquest": "; ",
    "ebsco": "; ",
    "jstor": "; ",
    "psycinfo": "; ",
    "eric": "; ",
    "econlit": "; ",
    # Other sciences
    "mathscinet": "; ",
    "inspec": "; ",
    "georef": "; ",
    "cab": "; ",
    "cinahl": "; ",
    "embase": "; ",
    "cochrane": "; ",
    # Generic formats
    "ris": "; ",
    "bibtex": " and ",
    "bib": " and ",
    "endnote": "; ",
    "zotero": "; ",
    "csv": "; ",
    "crossref": "; ",
    "orcid": "; ",
}


# =============================================================================
# BIBLIO BASE CLASS
# =============================================================================

class BiblioBase:
    """
    Base class providing shared initialization and helper methods for
    BiblioAnalysis and BiblioGroupAnalysis.
    
    This class is not meant to be instantiated directly. It provides:
    - Common initialization logic via _init_common()
    - Helper methods for saving tables and plots (_save_table, _save_plot)
    - Column lookup helper (_get_column)
    - Caching support for expensive computations
    """
    
    # Cache configuration
    _cache_enabled: bool = True
    _instance_cache: Dict = None
    
    def _init_cache(self) -> None:
        """Initialize the instance cache."""
        self._instance_cache = {}
        self._cache_enabled = True
    
    def _get_cached(
        self,
        cache_key: str,
        compute_func: Callable,
        *args,
        **kwargs,
    ):
        """
        Get a cached result or compute and cache it.
        
        Parameters
        ----------
        cache_key : str
            Unique key for this computation.
        compute_func : callable
            Function to compute the result if not cached.
        *args, **kwargs :
            Arguments passed to compute_func.
            
        Returns
        -------
        Any
            The cached or newly computed result.
        """
        if self._instance_cache is None:
            self._init_cache()
        
        if not self._cache_enabled:
            return compute_func(*args, **kwargs)
        
        if cache_key not in self._instance_cache:
            self._instance_cache[cache_key] = compute_func(*args, **kwargs)
        
        return self._instance_cache[cache_key]
    
    def clear_cache(self, prefix: Optional[str] = None) -> int:
        """
        Clear the instance cache.
        
        Parameters
        ----------
        prefix : str, optional
            If provided, only clear keys starting with this prefix.
            
        Returns
        -------
        int
            Number of items cleared.
        """
        if self._instance_cache is None:
            return 0
        
        if prefix is None:
            count = len(self._instance_cache)
            self._instance_cache.clear()
            return count
        
        keys_to_remove = [k for k in self._instance_cache if k.startswith(prefix)]
        for k in keys_to_remove:
            del self._instance_cache[k]
        return len(keys_to_remove)
    
    def set_cache_enabled(self, enabled: bool) -> None:
        """Enable or disable instance caching."""
        self._cache_enabled = enabled
    
    def get_cache_info(self) -> Dict:
        """
        Get cache information.
        
        Returns
        -------
        dict
            Cache statistics including size and enabled status.
        """
        if self._instance_cache is None:
            return {"size": 0, "enabled": self._cache_enabled, "keys": []}
        return {
            "size": len(self._instance_cache),
            "enabled": self._cache_enabled,
            "keys": list(self._instance_cache.keys()),
        }
    
    def _make_cache_key(self, prefix: str, *args, **kwargs) -> str:
        """
        Generate a cache key from prefix and arguments.
        
        Parameters
        ----------
        prefix : str
            Cache key prefix (e.g., "cooc", "network").
        *args, **kwargs :
            Arguments to include in the key.
            
        Returns
        -------
        str
            A unique cache key.
        """
        import hashlib
        
        components = [prefix]
        
        for arg in args:
            if isinstance(arg, pd.DataFrame):
                # Use shape and a hash of column names
                components.append(f"df:{arg.shape}:{hash(tuple(arg.columns))}")
            elif isinstance(arg, (list, tuple)):
                if all(isinstance(x, str) for x in arg):
                    components.append(str(sorted(arg)))
                else:
                    components.append(str(arg))
            elif arg is None:
                components.append("None")
            else:
                components.append(str(arg))
        
        for k, v in sorted(kwargs.items()):
            if isinstance(v, pd.DataFrame):
                components.append(f"{k}=df:{v.shape}")
            elif v is None:
                components.append(f"{k}=None")
            else:
                components.append(f"{k}={v}")
        
        combined = "|".join(components)
        return hashlib.md5(combined.encode()).hexdigest()[:16]
    
    def _init_common(
        self,
        f_name: Optional[str] = None,
        db: str = "",
        df: Optional[pd.DataFrame] = None,
        res_folder: Optional[str] = "results",
        output_lang: str = "en",
        preprocess_level: int = 0,
        exclude_list_kw: Optional[List[str]] = None,
        synonyms_kw: Optional[Dict[str, str]] = None,
        lemmatize_kw: bool = False,
        default_keywords: str = "author",
        combine_with_index_keywords: bool = False,
        concept_df: Optional[pd.DataFrame] = None,
        concept_column: Optional[str] = None,
        asjc_map_df: Optional[pd.DataFrame] = None,
        lang_of_docs: str = "en",
        fancy_output: bool = False,
        label_docs: bool = True,
        dpi: int = 600,
        cmap: str = "viridis",
        cmap_disc: str = "tab10",
        default_color: str = "lightblue",
        extra_stopwords: Optional[List[str]] = None,
        specific_stopword_categories: Optional[List[str]] = None,
    ) -> None:
        """
        Initialize common attributes for bibliometric analysis.
        
        This method handles all shared initialization logic between
        BiblioAnalysis and BiblioGroupAnalysis.
        """
        # Initialize cache
        self._init_cache()
        
        # Lazy imports to avoid circular dependencies
        from biblium import readbib, utilsbib
        
        # Store database and language settings
        self.db = db.lower()
        self.output_lang = output_lang
        self.ldf = lambda x: utilsbib.ldf(x, l=output_lang)
        
        # Store plotting configuration
        self.dpi = dpi
        self.cmap = cmap
        self.cmap_disc = cmap_disc
        self.default_color = default_color
        
        # Store enrichment data
        self.concept_df = concept_df
        
        # Load data
        if df is not None:
            self.df = df.copy() if hasattr(df, "copy") else df
        elif f_name is not None:
            self.df = readbib.read_bibfile(f_name, self.db)
        else:
            print(
                self.ldf("No dataset has been provided."),
                self.ldf("The bibliometric analysis cannot be performed."),
            )
            return
        
        self.n = len(self.df)
        
        # Setup database-specific separator
        self.default_separator = SEPARATOR_MAP.get(self.db, "; ")
        
        # Load mappings
        mapping_path = os.path.join(
            os.path.dirname(__file__), "additional files", "mappings.xlsx"
        )
        mapping_df = pd.read_excel(mapping_path, sheet_name="mapping")
        alias_df = pd.read_excel(mapping_path, sheet_name="alias")
        self.mapping = utilsbib.reconstruct_mapping(mapping_df, alias_df)
        
        # Add document labels
        if label_docs:
            self.df["Doc ID"] = [f"Doc {i}" for i in range(1, self.n + 1)]
        
        # Create source abbreviations
        if "Source title" in self.df.columns:
            if "Abbreviated Source Title" not in self.df.columns:
                self.df["Abbreviated Source Title"] = self.df["Source title"].map(
                    utilsbib.abbreviate_words
                )
            self.sources_abb_dict = self.df.set_index("Source title")[
                "Abbreviated Source Title"
            ].to_dict()
        else:
            self.sources_abb_dict = {}
        
        # Setup output folders
        if res_folder is not None:
            self.res_folder = os.path.join(os.getcwd(), res_folder)
            sub_folders = ["plots", "tables", "reports", "networks", "relations"]
            folders = [os.path.join(self.res_folder, s) for s in sub_folders]
            utilsbib.make_folders(folders)
            self.cond_formatting = fancy_output
            self.autofit = fancy_output
        else:
            self.res_folder = None
            self.cond_formatting = False
            self.autofit = False
        
        # Initialize relations dict
        self.relations = {}
        
        # =====================================================================
        # PREPROCESSING LEVELS
        # =====================================================================
        
        # Level 1: Basic enrichment
        if preprocess_level >= 1:
            if "Cited by" in self.df.columns:
                self.df["Percent Rank Cited by"] = (
                    pd.to_numeric(self.df["Cited by"], errors="coerce")
                    .rank(pct=True, method="average")
                    .mul(100)
                    .round(2)
                )
            
            self.df = utilsbib.add_ca_country_df(self.df, self.db)
            self.missings_df, self.missings = utilsbib.check_missing_values(self.df)
            self.df = utilsbib.add_document_labels_abbrev(self.df)
            
            if "Doc ID" in self.df.columns:
                doc_id_index = self.df.set_index("Doc ID")
                if "Document Short Label" in doc_id_index.columns:
                    self.id_short_label_dict = doc_id_index["Document Short Label"].to_dict()
                if "Document Label" in doc_id_index.columns:
                    self.id_label_dict = doc_id_index["Document Label"].to_dict()
            
            if self.db == "scopus" and "Affiliations" in self.df.columns:
                self.df, self.country_collab_matrix = utilsbib.extract_countries_from_affiliations(
                    self.df, aff_column="Affiliations"
                )
        
        # Level 2: Keywords and text processing
        if preprocess_level >= 2:
            self.process_keywords(
                exclude_list=exclude_list_kw,
                synonyms=synonyms_kw,
                lemmatize=lemmatize_kw,
            )
            self.process_text_vars(
                stopwords_file=utilsbib.stopwords_file,
                lang=lang_of_docs,
                extra_stopwords=extra_stopwords,
                exclude_specific_stopwords=specific_stopword_categories,
            )
            self.get_country_collaboration()
            self.df = utilsbib.build_combined_text(
                self.df,
                include_index_keywords=combine_with_index_keywords,
            )
            
            if concept_df is not None:
                if concept_column is None:
                    concept_column = utilsbib.first_existing(
                        self.df,
                        [
                            "Processed Combined Text",
                            "Combined Text",
                            "Processed Abstract",
                            "Processed Title",
                            "Abstract",
                            "Title",
                            "Processed Author Keywords",
                            "Author Keywords",
                        ],
                    )
                self.df = utilsbib.add_concept_indicators(
                    self.df, concept_df, text_col=concept_column
                )
        
        # Level 3: Science mappings
        if preprocess_level >= 3:
            if self.db == "scopus":
                cited_sciences = preprocess_level == 4
                self.add_sciences_scopus(
                    asjc_map_df=asjc_map_df,
                    cited_sciences=cited_sciences,
                )
            
            self.df["Author and Index Keywords"] = utilsbib.merge_keywords_columns(
                self.df,
                author_col="Author Keywords",
                index_col="Index Keywords",
                sep=self.default_separator,
            )
            self.df = utilsbib.merge_text_columns(
                self.df,
                title_col="Processed Title",
                abstract_col="Processed Abstract",
                author_col="Processed Author Keywords",
                index_col="Processed Index Keywords",
            )
        
        # Level 4: Interdisciplinarity
        if preprocess_level >= 4:
            if hasattr(self, "cited_sciences_df"):
                self.compute_interdisciplinarity_entropy()
        
        # Describe columns
        self.describe_columns()
        
        # Setup default keyword variable
        if default_keywords == "author":
            self.kw_var = (
                "Processed Author Keywords"
                if "Processed Author Keywords" in self.df.columns
                else "Author Keywords"
            )
        elif default_keywords == "index":
            self.kw_var = (
                "Processed Index Keywords"
                if "Processed Index Keywords" in self.df.columns
                else "Index Keywords"
            )
        elif default_keywords in ["both", "author and index"]:
            if "Author and Index Keywords" not in self.df.columns:
                self.df["Author and Index Keywords"] = utilsbib.merge_keywords_columns(
                    self.df,
                    author_col="Author Keywords",
                    index_col="Index Keywords",
                    sep=self.default_separator,
                )
            self.kw_var = (
                "Processed Author and Index Keywords"
                if "Processed Author and Index Keywords" in self.df.columns
                else "Author and Index Keywords"
            )
        else:
            self.kw_var = None
        
        # Extract author mappings for Scopus
        if ("Author full names" in self.df.columns) and self.db == "scopus":
            self.id_to_author, self.author_to_id = utilsbib.extract_author_mappings(
                self.df, "Author full names"
            )
        
        # Save missing values report
        if self.res_folder is not None and hasattr(self, "missings_df"):
            self._save_table(self.missings_df, "missing values")
    
    # =========================================================================
    # HELPER METHODS
    # =========================================================================
    
    def _save_table(
        self,
        df: pd.DataFrame,
        name: str,
        subfolder: str = "tables",
    ) -> None:
        """
        Save DataFrame to Excel if res_folder is set.
        
        Parameters
        ----------
        df : DataFrame
            Data to save.
        name : str
            Filename (without extension).
        subfolder : str
            Subfolder within res_folder.
        """
        if self.res_folder is not None and df is not None:
            from biblium import utilsbib
            utilsbib.to_excel_fancy(
                df,
                f_name=os.path.join(self.res_folder, subfolder, f"{name}.xlsx"),
                autofit=getattr(self, "autofit", False),
                conditional_formatting=getattr(self, "cond_formatting", False),
            )
    
    def _save_plot(self, filename_base: str, subfolder: str = "plots") -> None:
        """
        Save current matplotlib figure if res_folder is set.
        
        Parameters
        ----------
        filename_base : str
            Filename without extension.
        subfolder : str
            Subfolder within res_folder.
        """
        if self.res_folder is not None:
            from biblium import utilsbib
            path = os.path.join(self.res_folder, subfolder, filename_base)
            utilsbib.save_plot(path, dpi=self.dpi)
    
    def _get_column(
        self,
        candidates: Union[str, List[str]],
        required: bool = True,
    ) -> Optional[str]:
        """
        Find the first available column from candidates.
        
        Parameters
        ----------
        candidates : str or list
            Single column name or list of candidates in priority order.
        required : bool
            If True, raise error when no column found.
            
        Returns
        -------
        str or None
            Found column name.
        """
        if isinstance(candidates, str):
            candidates = [candidates]
        
        for col in candidates:
            if col in self.df.columns:
                return col
        
        if required:
            raise ValueError(f"None of the columns found: {candidates}")
        return None
    
    def _count_entity(
        self,
        column: Union[str, List[str]],
        item_label: str,
        count_type: str = "list",
        attr_name: Optional[str] = None,
        binary_attr_name: Optional[str] = None,
        save_name: Optional[str] = None,
        sep: Optional[str] = None,
        top_n: int = 0,
        rename_dict: Optional[Dict[str, str]] = None,
        translated_column_name: Optional[str] = None,
        ngram_range: tuple = (1, 1),
        post_process: Optional[callable] = None,
        **kwargs,
    ) -> pd.DataFrame:
        """
        Generic entity counting method.
        
        This is the core helper that powers all count_* methods, eliminating
        repetitive code patterns.
        
        Parameters
        ----------
        column : str or list
            Column name(s) to count. If list, uses first available.
        item_label : str
            Label for the item column in output (e.g., "Source", "Keyword").
        count_type : {"single", "list", "text"}
            How to count values in the column.
        attr_name : str, optional
            Attribute name to store result (e.g., "sources_counts_df").
            If None, derives from item_label.
        binary_attr_name : str, optional
            Attribute name for binary indicators (e.g., "binary_sources_df").
        save_name : str, optional
            Filename for saving (without extension). If None, derives from item_label.
        sep : str, optional
            Separator for list values. Uses self.default_separator if None.
        top_n : int
            If > 0, create binary indicators for top N items.
        rename_dict : dict, optional
            Mapping for translated item names.
        translated_column_name : str, optional
            Column name for translated items.
        ngram_range : tuple
            N-gram range for text count_type.
        post_process : callable, optional
            Function to apply to result DataFrame before saving.
        **kwargs
            Additional arguments for binary indicator creation.
            
        Returns
        -------
        DataFrame
            Counts DataFrame with ranks and proportions.
        """
        from biblium import utilsbib
        
        # Resolve column name
        if isinstance(column, list):
            col = self._get_column(column, required=False)
            if col is None:
                # Return empty DataFrame if column not found
                return pd.DataFrame(columns=[item_label, "Number of documents"])
        else:
            col = column
            if col not in self.df.columns:
                return pd.DataFrame(columns=[item_label, "Number of documents"])
        
        # Use default separator if not specified
        if sep is None:
            sep = getattr(self, "default_separator", "; ")
        
        # Build count_occurrences kwargs
        count_kwargs = {
            "count_type": count_type,
            "item_column_name": item_label,
            "sep": sep,
        }
        
        if count_type == "text":
            count_kwargs["ngram_range"] = ngram_range
        
        if rename_dict is not None:
            count_kwargs["rename_dict"] = rename_dict
            if translated_column_name:
                count_kwargs["translated_column_name"] = translated_column_name
        
        # Perform counting
        result_df = utilsbib.count_occurrences(self.df, col, **count_kwargs)
        
        # Apply post-processing if provided
        if post_process is not None:
            result_df = post_process(result_df)
        
        # Store as attribute
        if attr_name is None:
            attr_name = f"{item_label.lower().replace(' ', '_').replace('-', '_')}_counts_df"
        setattr(self, attr_name, result_df)
        
        # Create binary indicators if requested (with caching)
        if top_n > 0 and not result_df.empty:
            top_items = result_df[item_label].head(top_n).tolist()
            value_type = "string" if count_type == "single" else count_type
            
            # Check cache for binary indicators
            cache_key = None
            indicators_dict = None
            if hasattr(self, '_instance_cache') and self._instance_cache is not None and self._cache_enabled:
                cache_key = self._make_cache_key(
                    "binary", col, tuple(top_items[:20]) if len(top_items) > 20 else tuple(top_items),
                    value_type=value_type
                )
                indicators_dict = self._instance_cache.get(cache_key)
            
            if indicators_dict is None:
                _, indicators_dict = utilsbib.match_items_and_compute_binary_indicators(
                    df=self.df,
                    col=col,
                    items_of_interest=top_items,
                    value_type=value_type,
                    separator=sep,
                    **kwargs,
                )
                if cache_key is not None and self._instance_cache is not None:
                    self._instance_cache[cache_key] = indicators_dict
            
            if binary_attr_name is None:
                binary_attr_name = f"binary_{item_label.lower().replace(' ', '_').replace('-', '_')}_df"
            setattr(self, binary_attr_name, indicators_dict.get("binary"))
        
        # Save to Excel
        if save_name is None:
            save_name = f"{item_label.lower()} counts"
        self._save_table(result_df, save_name)
        
        return result_df
    
    def _get_entity_stats(
        self,
        column: Union[str, List[str]],
        item_label: str,
        count_method_name: str,
        attr_name: str,
        indicators_attr_name: str,
        save_name: str,
        value_type: str = "list",
        top_n: int = 100,
        post_process: Optional[Callable[[pd.DataFrame], pd.DataFrame]] = None,
        **kwargs,
    ) -> pd.DataFrame:
        """
        Generic helper for get_*_stats methods.
        
        This consolidates the common pattern used by get_sources_stats,
        get_author_keywords_stats, get_authors_stats, etc.
        
        Parameters
        ----------
        column : str or list of str
            Column name(s) to analyze (first existing will be used).
        item_label : str
            Label for items in the result (e.g., "Source", "Keyword").
        count_method_name : str
            Name of the count method to call (e.g., "count_sources").
        attr_name : str
            Attribute name to store results (e.g., "sources_stats_df").
        indicators_attr_name : str
            Attribute name for indicators (e.g., "sources_indicators").
        save_name : str
            Name for saving to Excel.
        value_type : str
            "string" for single values, "list" for delimited lists.
        top_n : int
            Number of top items to analyze (default 100).
        post_process : callable, optional
            Function to apply to result DataFrame.
        **kwargs :
            Additional arguments for get_entity_stats.
        
        Returns
        -------
        DataFrame
            Statistics for the entity.
        """
        from biblium import utilsbib
        
        # Resolve column
        if isinstance(column, str):
            col = column if column in self.df.columns else None
        else:
            col = self._get_column(column, required=False)
        
        if col is None:
            return pd.DataFrame()
        
        # Get count method
        count_method = getattr(self, count_method_name, None)
        if count_method is None:
            return pd.DataFrame()
        
        # Call get_entity_stats
        stats_df, indicators = utilsbib.get_entity_stats(
            self.df,
            col,
            item_label,
            count_method=count_method,
            value_type=value_type,
            sep=getattr(self, "default_separator", "; "),
            top_n=top_n,
            **kwargs,
        )
        
        # Apply post-processing if provided
        if post_process is not None and stats_df is not None:
            stats_df = post_process(stats_df)
        
        # Store results
        setattr(self, attr_name, stats_df)
        setattr(self, indicators_attr_name, indicators)
        
        # Save
        self._save_table(stats_df, save_name)
        
        return stats_df
    
    # =========================================================================
    # PLACEHOLDER METHODS (implemented in subclasses)
    # =========================================================================
    
    def describe_columns(self) -> None:
        """Describe DataFrame columns. Override in subclass."""
        pass
    
    def process_keywords(self, **kwargs) -> None:
        """Process keywords. Override in subclass."""
        pass
    
    def process_text_vars(self, **kwargs) -> None:
        """Process text variables. Override in subclass."""
        pass
    
    def get_country_collaboration(self, **kwargs) -> None:
        """Get country collaboration. Override in subclass."""
        pass
    
    def add_sciences_scopus(self, **kwargs) -> None:
        """Add Scopus sciences. Override in subclass."""
        pass
    
    def compute_interdisciplinarity_entropy(self, **kwargs) -> None:
        """Compute interdisciplinarity. Override in subclass."""
        pass



# -*- coding: utf-8 -*-
"""
Group Logistic Regression Panel
================================
GUI panel for statistical logistic regression analysis of group membership
with coefficients, odds ratios, p-values, and significance indicators.
"""

import tkinter as tk
from tkinter import ttk, messagebox, filedialog
from typing import Dict, List, Optional, Any
import pandas as pd
import numpy as np

from biblium.gui.panels.base import BasePanel
from biblium.gui.widgets.cards import Card, StatsCard, CardGrid
from biblium.gui.widgets.forms import LabeledCombobox, LabeledCheckbox, LabeledEntry
from biblium.gui.widgets.buttons import ActionButton
from biblium.gui.widgets.plots import PlotFrame
from biblium.gui.widgets.tables import DataTable
from biblium.gui.config import FONTS, get_theme
from biblium.gui.core.events import EventBus

try:
    import matplotlib.pyplot as plt
    HAS_MATPLOTLIB = True
except ImportError:
    HAS_MATPLOTLIB = False

event_bus = EventBus()


class GroupLogisticPanel(BasePanel):
    """
    Panel for statistical logistic regression analysis of group membership.
    
    Uses BiblioGroupClassifier.logistic_regression_analysis() to fit logistic
    regression models with statsmodels and display coefficients, odds ratios,
    p-values, and significance indicators.
    
    Features:
    - Text-based features (keywords, abstracts) or custom design matrix
    - Configurable predictors (top_n, regex filters, items of interest)
    - Per-group logistic regression models
    - Coefficients with standard errors and confidence intervals
    - Odds ratios with interpretation
    - P-values with significance highlighting
    - Direction indicators (↑/↓ arrows for significant effects)
    - Model statistics (AIC, BIC, pseudo R²)
    - Export to Excel with formatting
    """
    
    title = "📉 Logistic Regression"
    description = "Statistical logistic regression with coefficients and p-values"
    
    def __init__(self, parent, theme: str = "light", bib=None, **kwargs):
        self.bib = bib
        self._bib_group = None
        self._results = None
        self._group_vars = {}  # Group name -> BooleanVar
        super().__init__(parent, theme=theme, bib=bib, **kwargs)
    
    @property
    def bib_group(self):
        """Get the BiblioGroup object from workspace or local storage."""
        if hasattr(self, '_bib_group') and self._bib_group is not None:
            return self._bib_group
        
        # Try to get from workspace
        workspace = self.master
        if hasattr(workspace, 'bib_group'):
            return workspace.bib_group
        if hasattr(workspace, '_shared_bib_group'):
            return workspace._shared_bib_group
        
        return None
    
    @bib_group.setter
    def bib_group(self, value):
        self._bib_group = value
    
    def _create_options(self):
        """Create options panel."""
        super()._create_options()
        
        # Dependent Variables Card (Groups)
        dv_card = Card(self.options_content, title="📊 Dependent Variables (Groups)", theme=self.theme_name)
        dv_card.pack(fill=tk.X, padx=8, pady=8)
        
        tk.Label(dv_card.content, text="Select groups to analyze:", 
                font=FONTS.get_font("small"), bg=self.theme["bg_card"],
                fg=self.theme["text_muted"]).pack(anchor=tk.W)
        
        # Scrollable frame for group checkboxes
        self.groups_frame = tk.Frame(dv_card.content, bg=self.theme["bg_card"])
        self.groups_frame.pack(fill=tk.X, pady=4)
        
        # Buttons for select all/none
        btn_frame = tk.Frame(dv_card.content, bg=self.theme["bg_card"])
        btn_frame.pack(fill=tk.X, pady=4)
        tk.Button(btn_frame, text="Select All", command=self._select_all_groups,
                 font=FONTS.get_font("small")).pack(side=tk.LEFT, padx=2)
        tk.Button(btn_frame, text="Select None", command=self._select_no_groups,
                 font=FONTS.get_font("small")).pack(side=tk.LEFT, padx=2)
        
        # Independent Variables Card
        iv_card = Card(self.options_content, title="📝 Independent Variables", theme=self.theme_name)
        iv_card.pack(fill=tk.X, padx=8, pady=8)
        
        # Text column for term extraction
        self.text_col_var = tk.StringVar(value="Author Keywords")
        text_combo = LabeledCombobox(
            iv_card.content, label="Text column:",
            values=["Author Keywords", "Index Keywords", "Abstract", "Title"],
            variable=self.text_col_var, theme=self.theme_name,
        )
        text_combo.pack(fill=tk.X, pady=4)
        self.text_combo = text_combo
        
        # Top N terms
        topn_frame = tk.Frame(iv_card.content, bg=self.theme["bg_card"])
        topn_frame.pack(fill=tk.X, pady=4)
        tk.Label(topn_frame, text="Top N terms:", font=FONTS.get_font("body"),
                bg=self.theme["bg_card"], fg=self.theme["text_primary"]).pack(side=tk.LEFT)
        self.topn_var = tk.StringVar(value="50")
        tk.Entry(topn_frame, textvariable=self.topn_var,
                font=FONTS.get_font("body"), width=8).pack(side=tk.LEFT, padx=8)
        tk.Label(topn_frame, text="(0 = all)", font=FONTS.get_font("small"),
                bg=self.theme["bg_card"], fg=self.theme["text_muted"]).pack(side=tk.LEFT)
        
        # Include/Exclude regex
        tk.Label(iv_card.content, text="Filter terms (optional):", 
                font=FONTS.get_font("small"), bg=self.theme["bg_card"],
                fg=self.theme["text_muted"]).pack(anchor=tk.W, pady=(8, 0))
        
        inc_frame = tk.Frame(iv_card.content, bg=self.theme["bg_card"])
        inc_frame.pack(fill=tk.X, pady=2)
        tk.Label(inc_frame, text="Include regex:", font=FONTS.get_font("body"),
                bg=self.theme["bg_card"], fg=self.theme["text_primary"], width=12, anchor=tk.W).pack(side=tk.LEFT)
        self.include_var = tk.StringVar(value="")
        tk.Entry(inc_frame, textvariable=self.include_var,
                font=FONTS.get_font("body")).pack(side=tk.LEFT, fill=tk.X, expand=True)
        
        exc_frame = tk.Frame(iv_card.content, bg=self.theme["bg_card"])
        exc_frame.pack(fill=tk.X, pady=2)
        tk.Label(exc_frame, text="Exclude regex:", font=FONTS.get_font("body"),
                bg=self.theme["bg_card"], fg=self.theme["text_primary"], width=12, anchor=tk.W).pack(side=tk.LEFT)
        self.exclude_var = tk.StringVar(value="")
        tk.Entry(exc_frame, textvariable=self.exclude_var,
                font=FONTS.get_font("body")).pack(side=tk.LEFT, fill=tk.X, expand=True)
        
        # Items of interest (specific terms)
        tk.Label(iv_card.content, text="Specific terms (optional, one per line):", 
                font=FONTS.get_font("small"), bg=self.theme["bg_card"],
                fg=self.theme["text_muted"]).pack(anchor=tk.W, pady=(8, 0))
        
        self.items_text = tk.Text(iv_card.content, height=4, font=FONTS.get_font("body"))
        self.items_text.pack(fill=tk.X, pady=4)
        
        # Significance Levels Card
        sig_card = Card(self.options_content, title="📊 Display Options", theme=self.theme_name)
        sig_card.pack(fill=tk.X, padx=8, pady=8)
        
        tk.Label(sig_card.content, text="Significance thresholds used for highlighting:", 
                font=FONTS.get_font("small"), bg=self.theme["bg_card"],
                fg=self.theme["text_muted"]).pack(anchor=tk.W)
        
        sig_frame = tk.Frame(sig_card.content, bg=self.theme["bg_card"])
        sig_frame.pack(fill=tk.X, pady=4)
        
        # Color legend
        legend_items = [
            ("p ≤ 0.001", "#006400", "↑↑↑ / ↓↓↓"),
            ("p ≤ 0.01", "#228B22", "↑↑ / ↓↓"),
            ("p ≤ 0.05", "#66CDAA", "↑ / ↓"),
            ("p ≤ 0.10", "#98FB98", ""),
        ]
        
        for threshold, color, arrows in legend_items:
            row = tk.Frame(sig_frame, bg=self.theme["bg_card"])
            row.pack(fill=tk.X, pady=1)
            color_box = tk.Label(row, text="   ", bg=color, width=3)
            color_box.pack(side=tk.LEFT, padx=4)
            tk.Label(row, text=f"{threshold}", font=FONTS.get_font("small"),
                    bg=self.theme["bg_card"], fg=self.theme["text_primary"], width=10).pack(side=tk.LEFT)
            if arrows:
                tk.Label(row, text=arrows, font=FONTS.get_font("small"),
                        bg=self.theme["bg_card"], fg=self.theme["text_muted"]).pack(side=tk.LEFT)
        
        # Action Buttons
        # LLM Configuration (v2.11)
        self._create_llm_config_card()
        
        btn_frame = tk.Frame(self.options_content, bg=self.theme["bg_secondary"])
        btn_frame.pack(fill=tk.X, padx=8, pady=16)
        
        ActionButton(
            btn_frame, text="Run Analysis", icon="▶️",
            command=self._run_analysis, theme=self.theme_name,
        ).pack(fill=tk.X, pady=4)
        
        ActionButton(
            btn_frame, text="Export Results", icon="💾",
            command=self._export_results, theme=self.theme_name,
        ).pack(fill=tk.X, pady=4)
    
    def _create_results(self):
        """Create results panel."""
        super()._create_results()
        
        # Create notebook with permanent Info tab
        self.results_notebook = ttk.Notebook(self.results_content)
        self.results_notebook.pack(fill=tk.BOTH, expand=True)
        
        # Results tab
        self.results_tab = tk.Frame(self.results_notebook, bg=self.theme["bg_card"])
        self.results_notebook.add(self.results_tab, text="📊 Results")
        
        # Info tab (always visible)
        self.info_frame = tk.Frame(self.results_notebook, bg=self.theme["bg_card"])
        self.results_notebook.add(self.info_frame, text="ℹ️ Info")
        self._create_info_content(self.info_frame)
        
        # Show placeholder in results tab
        self._show_results_placeholder()
    
    def _show_results_placeholder(self):
        """Show placeholder in results tab."""
        # Check if results_tab still exists
        if not hasattr(self, 'results_tab'):
            return
        try:
            if not self.results_tab.winfo_exists():
                return
        except Exception:
            return
        
        for widget in self.results_tab.winfo_children():
            widget.destroy()
        tk.Label(
            self.results_tab,
            text="Click 'Run' to see results here.\nSee Info tab for documentation.",
            font=FONTS.get_font("body"),
            bg=self.theme["bg_card"],
            fg=self.theme["text_muted"],
            justify=tk.CENTER,
        ).pack(expand=True)

    def _select_all_groups(self):
        """Select all groups."""
        for var in self._group_vars.values():
            var.set(True)
    
    def _select_no_groups(self):
        """Deselect all groups."""
        for var in self._group_vars.values():
            var.set(False)
    
    def _populate_groups(self):
        """Populate group checkboxes from group_matrix."""
        for widget in self.groups_frame.winfo_children():
            widget.destroy()
        self._group_vars.clear()
        
        # Check for bib_group first, then fall back to bib.group_matrix
        group_matrix = None
        if self.bib_group is not None and hasattr(self.bib_group, 'group_matrix') and self.bib_group.group_matrix is not None:
            group_matrix = self.bib_group.group_matrix
        elif self.bib is not None and hasattr(self.bib, 'group_matrix') and self.bib.group_matrix is not None:
            group_matrix = self.bib.group_matrix
        
        if group_matrix is None or len(group_matrix.columns) == 0:
            tk.Label(self.groups_frame, text="No groups defined. Set up groups first.",
                    font=FONTS.get_font("small"), bg=self.theme["bg_card"],
                    fg=self.theme["text_muted"]).pack(anchor=tk.W)
            return
        
        for col in group_matrix.columns:
            var = tk.BooleanVar(value=True)
            self._group_vars[col] = var
            cb = tk.Checkbutton(self.groups_frame, text=col, variable=var,
                               bg=self.theme["bg_card"], font=FONTS.get_font("body"))
            cb.pack(anchor=tk.W)
    
    def _populate_columns(self):
        """Populate column dropdowns."""
        # Get dataframe from bib_group or bib
        df = None
        if self.bib_group is not None and hasattr(self.bib_group, 'df'):
            df = self.bib_group.df
        elif self.bib is not None and hasattr(self.bib, 'df'):
            df = self.bib.df
        
        if df is None:
            return
        
        cols = list(df.columns)
        
        # Text columns - prefer keyword columns
        text_cols = []
        for c in cols:
            c_lower = c.lower()
            if any(kw in c_lower for kw in ['keyword', 'abstract', 'title', 'description']):
                text_cols.append(c)
        
        if not text_cols:
            text_cols = cols[:10]
        
        self.text_combo.combobox['values'] = text_cols
        
        # Set default to "Processed Abstract" if available
        if 'Processed Abstract' in text_cols:
            self.text_col_var.set('Processed Abstract')
        else:
            # Fall back to keyword column if available
            for col in text_cols:
                if 'keyword' in col.lower():
                    self.text_col_var.set(col)
                    break
    
    def _get_selected_groups(self) -> List[str]:
        """Get list of selected group names."""
        return [name for name, var in self._group_vars.items() if var.get()]
    
    def _run_analysis(self):
        """Run the logistic regression analysis."""
        # Get dataframe and group_matrix from bib_group or bib
        df = None
        group_matrix = None
        
        if self.bib_group is not None:
            if hasattr(self.bib_group, 'df'):
                df = self.bib_group.df
            if hasattr(self.bib_group, 'group_matrix'):
                group_matrix = self.bib_group.group_matrix
        
        if df is None and self.bib is not None and hasattr(self.bib, 'df'):
            df = self.bib.df
        if group_matrix is None and self.bib is not None and hasattr(self.bib, 'group_matrix'):
            group_matrix = self.bib.group_matrix
        
        if df is None:
            messagebox.showwarning("No Data", "Please load a dataset first.")
            return
        
        if group_matrix is None or len(group_matrix.columns) == 0:
            messagebox.showwarning("No Groups", "Please set up groups first (use Group Setup panel).")
            return
        
        selected_groups = self._get_selected_groups()
        if not selected_groups:
            messagebox.showwarning("No Groups Selected", "Please select at least one group to analyze.")
            return
        
        self._show_loading("Running logistic regression analysis...")
        self.update_idletasks()
        
        try:
            import statsmodels.api as sm
            from sklearn.feature_extraction.text import CountVectorizer
            import re
            
            # Get settings
            text_column = self.text_col_var.get()
            
            try:
                top_n = int(self.topn_var.get())
            except ValueError:
                top_n = 50
            
            include_regex = self.include_var.get().strip() or None
            exclude_regex = self.exclude_var.get().strip() or None
            
            # Get items of interest
            items_text = self.items_text.get("1.0", tk.END).strip()
            items_of_interest = None
            if items_text:
                items_of_interest = [item.strip() for item in items_text.split('\n') if item.strip()]
            
            # Build term matrix from text column
            if text_column not in df.columns:
                self._show_error(f"Column '{text_column}' not found in data.")
                return
            
            texts = df[text_column].fillna("").astype(str)
            
            # Check if it's a keyword column (use special tokenizer)
            is_keyword_col = "keyword" in text_column.lower()
            
            if is_keyword_col:
                # Get separator from bib_group if available
                sep = "; "
                if self.bib_group is not None and hasattr(self.bib_group, 'default_separator'):
                    sep = self.bib_group.default_separator
                
                def keyword_tokenizer(text, _sep=sep):
                    if not text:
                        return []
                    return [t.strip() for t in str(text).split(_sep) if t.strip()]
                
                vec = CountVectorizer(
                    tokenizer=keyword_tokenizer,
                    token_pattern=None,
                    max_features=max(500, top_n * 2) if top_n > 0 else 1000,
                    binary=True,
                )
            else:
                vec = CountVectorizer(
                    max_features=max(500, top_n * 2) if top_n > 0 else 1000,
                    stop_words="english",
                    binary=True,
                )
            
            Xc = vec.fit_transform(texts)
            vocab = np.array(vec.get_feature_names_out())
            doc_counts = (Xc > 0).sum(axis=0).A1
            
            items_df = pd.DataFrame({"item": vocab, "doc_count": doc_counts})
            
            # Apply filters
            if include_regex:
                items_df = items_df[items_df["item"].str.contains(include_regex, regex=True, na=False)]
            if exclude_regex:
                items_df = items_df[~items_df["item"].str.contains(exclude_regex, regex=True, na=False)]
            
            # Select items
            if items_of_interest is not None:
                items_set = set(items_df["item"])
                selected_items = [it for it in items_of_interest if it in items_set]
                if not selected_items:
                    self._show_error("No items of interest found in vocabulary after filtering.")
                    return
                items = selected_items
            else:
                items_df = items_df.sort_values("doc_count", ascending=False)
                if top_n > 0:
                    items = items_df.head(top_n)["item"].tolist()
                else:
                    items = items_df["item"].tolist()
            
            if not items:
                self._show_error("No predictors remain after filtering.")
                return
            
            # Build design matrix
            X_dense = Xc.toarray()
            X_all = pd.DataFrame(X_dense, columns=vocab, index=df.index)
            X_design = X_all[items]
            X_design = sm.add_constant(X_design, has_constant="add")
            
            # Store for export
            self._df = df
            self._group_matrix = group_matrix
            
            # Fit logistic regression for each group
            results = {}
            
            for grp in selected_groups:
                if grp not in group_matrix.columns:
                    continue
                
                y = group_matrix[grp].values
                
                # Skip if only one class
                if len(np.unique(y)) < 2:
                    print(f"Skipping logistic regression for group '{grp}': only one class")
                    continue
                
                try:
                    model = sm.Logit(y, X_design).fit(disp=False)
                    coef_table = model.summary2().tables[1]
                    results[grp] = {"model": model, "summary": coef_table}
                except Exception as e:
                    print(f"Logit failed for group '{grp}': {e}")
            
            if not results:
                self._show_error("No results generated. Check that groups have variation (not all 0 or all 1).")
                return
            
            self._results = results
            self._display_results(results)
            
        except Exception as e:
            self._show_error(str(e))
            import traceback
            traceback.print_exc()
    
    def _display_results(self, results: Dict):
        """Display logistic regression results."""
        # Check if results_tab still exists
        if not hasattr(self, 'results_tab'):
            return
        try:
            if not self.results_tab.winfo_exists():
                return
        except Exception:
            return
        
        for widget in self.results_tab.winfo_children():
            widget.destroy()
        
        # Summary stats
        grid = CardGrid(self.results_tab, columns=4, theme=self.theme_name)
        grid.pack(fill=tk.X, pady=(0, 8))
        
        n_groups = len(results)
        total_sig = 0
        total_predictors = 0
        avg_pseudo_r2 = 0
        
        for grp, data in results.items():
            model = data["model"]
            coef_df = data["summary"]
            
            # Count significant predictors (excluding const)
            if "P>|z|" in coef_df.columns:
                p_vals = pd.to_numeric(coef_df["P>|z|"], errors="coerce")
                # Exclude const
                if "const" in coef_df.index:
                    p_vals = p_vals.drop("const", errors="ignore")
                sig_count = (p_vals <= 0.05).sum()
                total_sig += sig_count
                total_predictors += len(p_vals)
            
            avg_pseudo_r2 += model.prsquared
        
        avg_pseudo_r2 /= n_groups if n_groups > 0 else 1
        
        grid.add_card(StatsCard(grid, "Groups", str(n_groups), "📊", self.theme_name))
        grid.add_card(StatsCard(grid, "Predictors", str(total_predictors // n_groups if n_groups else 0), "📝", self.theme_name))
        grid.add_card(StatsCard(grid, "Significant (p≤0.05)", str(total_sig), "✓", self.theme_name))
        grid.add_card(StatsCard(grid, "Avg Pseudo R²", f"{avg_pseudo_r2:.3f}", "📈", self.theme_name))
        
        # Create notebook with one tab per group + summary
        notebook = ttk.Notebook(self.results_tab)
        notebook.pack(fill=tk.BOTH, expand=True, pady=8)
        
        # Summary tab
        summary_frame = tk.Frame(notebook, bg=self.theme["bg_card"])
        notebook.add(summary_frame, text="Summary")
        
        summary_rows = []
        for grp, data in results.items():
            model = data["model"]
            coef_df = data["summary"]
            
            # Count significant at different levels
            p_vals = pd.to_numeric(coef_df["P>|z|"], errors="coerce")
            if "const" in coef_df.index:
                p_vals_no_const = p_vals.drop("const", errors="ignore")
            else:
                p_vals_no_const = p_vals
            
            summary_rows.append({
                "Group": grp,
                "N_obs": model.nobs,
                "N_predictors": len(p_vals_no_const),
                "Sig_0.10": (p_vals_no_const <= 0.10).sum(),
                "Sig_0.05": (p_vals_no_const <= 0.05).sum(),
                "Sig_0.01": (p_vals_no_const <= 0.01).sum(),
                "Sig_0.001": (p_vals_no_const <= 0.001).sum(),
                "AIC": round(model.aic, 2),
                "BIC": round(model.bic, 2),
                "Pseudo_R2": round(model.prsquared, 4),
            })
        
        summary_df = pd.DataFrame(summary_rows)
        summary_table = DataTable(summary_frame, theme=self.theme_name)
        summary_table.pack(fill=tk.BOTH, expand=True)
        summary_table.set_data(summary_df)
        
        # Per-group tabs
        for grp, data in results.items():
            grp_frame = tk.Frame(notebook, bg=self.theme["bg_card"])
            notebook.add(grp_frame, text=grp[:15])

            

            # Info tab

            info_frame = tk.Frame(notebook, bg=self.theme["bg_card"])

            notebook.add(info_frame, text="ℹ️ Info")

            self._create_info_content(info_frame)
            
            self._create_group_tab(grp_frame, grp, data)
        
        # Store for export
        self._results_dict = results
    
    def _create_group_tab(self, parent, group_name: str, data: Dict):
        """Create a tab for a single group's results."""
        model = data["model"]
        coef_df = data["summary"].copy()
        
        # Add odds ratios and direction
        coef_df["OR"] = np.exp(coef_df["Coef."])
        
        # Add direction arrows
        if "Coef." in coef_df.columns and "P>|z|" in coef_df.columns:
            coef_vals = pd.to_numeric(coef_df["Coef."], errors="coerce")
            p_vals = pd.to_numeric(coef_df["P>|z|"], errors="coerce")
            
            direction = []
            for idx in coef_df.index:
                coef = coef_vals.get(idx, 0)
                p = p_vals.get(idx, 1)
                
                if idx == "const" or pd.isna(p):
                    direction.append("")
                elif p <= 0.001:
                    direction.append("↑↑↑" if coef > 0 else "↓↓↓")
                elif p <= 0.01:
                    direction.append("↑↑" if coef > 0 else "↓↓")
                elif p <= 0.05:
                    direction.append("↑" if coef > 0 else "↓")
                else:
                    direction.append("")
            
            coef_df["Direction"] = direction
        
        # Model stats at top
        stats_frame = tk.Frame(parent, bg=self.theme["bg_card"])
        stats_frame.pack(fill=tk.X, pady=8, padx=8)
        
        stats_text = f"N = {int(model.nobs)}  |  AIC = {model.aic:.2f}  |  BIC = {model.bic:.2f}  |  Pseudo R² = {model.prsquared:.4f}"
        tk.Label(stats_frame, text=stats_text, font=FONTS.get_font("body"),
                bg=self.theme["bg_card"], fg=self.theme["text_primary"]).pack(anchor=tk.W)
        
        # Coefficient table
        # Reorder columns for display
        display_cols = ["Coef.", "Std.Err.", "z", "P>|z|", "[0.025", "0.975]", "OR", "Direction"]
        display_cols = [c for c in display_cols if c in coef_df.columns]
        
        coef_display = coef_df[display_cols].copy()
        coef_display = coef_display.reset_index()
        coef_display.columns = ["Term"] + list(coef_display.columns[1:])
        
        # Round numeric columns
        for col in coef_display.columns:
            if col not in ["Term", "Direction"]:
                coef_display[col] = pd.to_numeric(coef_display[col], errors="coerce").round(4)
        
        # Create table with highlighting
        table_frame = tk.Frame(parent, bg=self.theme["bg_card"])
        table_frame.pack(fill=tk.BOTH, expand=True, padx=8, pady=8)
        
        # Create treeview with custom colors
        tree = ttk.Treeview(table_frame, columns=list(coef_display.columns), show="headings", height=20)
        
        for col in coef_display.columns:
            tree.heading(col, text=col)
            width = 80 if col not in ["Term"] else 150
            tree.column(col, width=width, anchor=tk.CENTER if col != "Term" else tk.W)
        
        # Configure tags for highlighting
        tree.tag_configure("sig_001", background="#006400", foreground="white")
        tree.tag_configure("sig_01", background="#228B22", foreground="white")
        tree.tag_configure("sig_05", background="#66CDAA", foreground="black")
        tree.tag_configure("sig_10", background="#98FB98", foreground="black")
        
        # Insert data with tags
        for _, row in coef_display.iterrows():
            p_val = row.get("P>|z|", 1)
            
            if pd.notna(p_val):
                if p_val <= 0.001:
                    tag = "sig_001"
                elif p_val <= 0.01:
                    tag = "sig_01"
                elif p_val <= 0.05:
                    tag = "sig_05"
                elif p_val <= 0.10:
                    tag = "sig_10"
                else:
                    tag = ""
            else:
                tag = ""
            
            values = [str(v) if pd.notna(v) else "" for v in row]
            tree.insert("", tk.END, values=values, tags=(tag,) if tag else ())
        
        # Scrollbars
        vsb = ttk.Scrollbar(table_frame, orient="vertical", command=tree.yview)
        hsb = ttk.Scrollbar(table_frame, orient="horizontal", command=tree.xview)
        tree.configure(yscrollcommand=vsb.set, xscrollcommand=hsb.set)
        
        tree.grid(row=0, column=0, sticky="nsew")
        vsb.grid(row=0, column=1, sticky="ns")
        hsb.grid(row=1, column=0, sticky="ew")
        
        table_frame.grid_rowconfigure(0, weight=1)
        table_frame.grid_columnconfigure(0, weight=1)
    
    def _export_results(self):
        """Export results to Excel."""
        if self._results is None:
            messagebox.showwarning("No Results", "Please run analysis first.")
            return
        
        filename = filedialog.asksaveasfilename(
            defaultextension=".xlsx",
            filetypes=[("Excel files", "*.xlsx")],
            title="Save Logistic Regression Results"
        )
        
        if not filename:
            return
        
        try:
            results = self._results
            
            with pd.ExcelWriter(filename, engine="xlsxwriter") as writer:
                workbook = writer.book
                
                # p-value formats
                p_formats = {
                    0.001: workbook.add_format({"bg_color": "#006400", "font_color": "#FFFFFF"}),
                    0.01: workbook.add_format({"bg_color": "#228B22", "font_color": "#FFFFFF"}),
                    0.05: workbook.add_format({"bg_color": "#66CDAA", "font_color": "#000000"}),
                    0.1: workbook.add_format({"bg_color": "#98FB98", "font_color": "#000000"}),
                }
                
                summary_rows = []
                
                for grp, data in results.items():
                    coef_df = data["summary"].copy()
                    model = data["model"]
                    
                    # Add odds ratios
                    coef_df["OR"] = np.exp(coef_df["Coef."])
                    
                    # Add direction arrows
                    if "Coef." in coef_df.columns and "P>|z|" in coef_df.columns:
                        coef_vals = pd.to_numeric(coef_df["Coef."], errors="coerce")
                        p_vals = pd.to_numeric(coef_df["P>|z|"], errors="coerce")
                        
                        direction = np.full(len(coef_df), "", dtype=object)
                        pos = coef_vals > 0
                        neg = coef_vals < 0
                        
                        band1 = (p_vals <= 0.05) & (p_vals > 0.01)
                        band2 = (p_vals <= 0.01) & (p_vals > 0.001)
                        band3 = p_vals <= 0.001
                        
                        direction[band1 & pos] = "↑"
                        direction[band1 & neg] = "↓"
                        direction[band2 & pos] = "↑↑"
                        direction[band2 & neg] = "↓↓"
                        direction[band3 & pos] = "↑↑↑"
                        direction[band3 & neg] = "↓↓↓"
                        
                        if "const" in coef_df.index:
                            direction[coef_df.index == "const"] = ""
                        
                        coef_df["Direction"] = direction
                    
                    # Write coefficients sheet
                    sheet_name = f"coeff {grp}"[:31]
                    coef_df.to_excel(writer, sheet_name=sheet_name, index=True)
                    
                    # Highlight p-values
                    ws = writer.sheets[sheet_name]
                    if "P>|z|" in coef_df.columns:
                        p_col_idx = coef_df.columns.get_loc("P>|z|") + 1
                        for row_idx, p_val in enumerate(coef_df["P>|z|"], start=1):
                            if pd.isna(p_val):
                                continue
                            for threshold, fmt in p_formats.items():
                                if p_val <= threshold:
                                    ws.write(row_idx, p_col_idx, p_val, fmt)
                                    break
                    
                    # Write statistics sheet
                    stats_df = pd.DataFrame({
                        "AIC": [model.aic],
                        "BIC": [model.bic],
                        "Pseudo R-squared": [model.prsquared],
                    })
                    stats_df.to_excel(writer, sheet_name=f"stats {grp}"[:31], index=False)
                    
                    # Summary row
                    p_vals_no_const = pd.to_numeric(coef_df["P>|z|"], errors="coerce")
                    if "const" in coef_df.index:
                        p_vals_no_const = p_vals_no_const.drop("const", errors="ignore")
                    
                    summary_rows.append({
                        "Group": grp,
                        "N_obs": int(model.nobs),
                        "N_predictors": len(p_vals_no_const),
                        "Sig_0.10": int((p_vals_no_const <= 0.10).sum()),
                        "Sig_0.05": int((p_vals_no_const <= 0.05).sum()),
                        "Sig_0.01": int((p_vals_no_const <= 0.01).sum()),
                        "Sig_0.001": int((p_vals_no_const <= 0.001).sum()),
                        "AIC": round(model.aic, 2),
                        "BIC": round(model.bic, 2),
                        "Pseudo_R2": round(model.prsquared, 4),
                    })
                
                # Write summary sheet
                summary_df = pd.DataFrame(summary_rows)
                summary_df.to_excel(writer, sheet_name="Summary", index=False)
            
            messagebox.showinfo("Success", f"Results saved to:\n{filename}")
            
        except Exception as e:
            messagebox.showerror("Error", f"Failed to export: {e}")
            import traceback
            traceback.print_exc()
    
    def _show_loading(self, message: str = "Processing..."):
        """Show loading indicator."""
        # Check if results_tab still exists
        if not hasattr(self, 'results_tab'):
            return
        try:
            if not self.results_tab.winfo_exists():
                return
        except Exception:
            return
        
        for widget in self.results_tab.winfo_children():
            widget.destroy()
        frame = tk.Frame(self.results_tab, bg=self.theme["bg_card"])
        frame.pack(expand=True)
        tk.Label(frame, text="⏳", font=("Segoe UI", 32), bg=self.theme["bg_card"],
                fg=self.theme["text_primary"]).pack(pady=(20, 10))
        tk.Label(frame, text=message, font=FONTS.get_font("body"), bg=self.theme["bg_card"],
                fg=self.theme["text_muted"], justify=tk.CENTER).pack()
    
    def _show_error(self, message: str):
        """Show error message."""
        # Check if results_tab still exists
        if not hasattr(self, 'results_tab'):
            return
        try:
            if not self.results_tab.winfo_exists():
                return
        except Exception:
            return
        
        for widget in self.results_tab.winfo_children():
            widget.destroy()
        error_color = self.theme.get("error", self.theme.get("danger", "#e74c3c"))
        tk.Label(self.results_tab, text=f"❌ Error\n\n{message}", font=FONTS.get_font("body"),
                bg=self.theme["bg_card"], fg=error_color, justify=tk.CENTER,
                wraplength=400).pack(expand=True)
    
    def set_bib(self, bib):
        """Set the bibliometric data object."""
        self.bib = bib
        self._populate_groups()
        self._populate_columns()
    
    def set_bib_group(self, bib_group):
        """Set the BiblioGroup object."""
        self._bib_group = bib_group
        self._populate_groups()
        self._populate_columns()
    
    def refresh(self):
        """Refresh the panel (called when switching to this panel)."""
        self._populate_groups()
        self._populate_columns()
    
    def _create_info_content(self, parent):
        """Create Info tab with documentation."""
        info_text = """
LOGISTIC REGRESSION
═══════════════════

Statistical model for predicting group membership.

MODEL OUTPUT
────────────
• Coefficients (β): Effect direction and strength
• Standard Errors: Coefficient precision
• Odds Ratios (OR): Multiplicative effect
• P-values: Statistical significance
• Confidence Intervals: Range of plausible values
• Direction Indicators: ↑ positive, ↓ negative

INTERPRETING COEFFICIENTS
─────────────────────────
• Positive β (↑): Increases group probability
• Negative β (↓): Decreases group probability
• Magnitude: Effect strength

ODDS RATIOS
───────────
• OR = exp(β)
• OR = 1.0: No effect
• OR > 1.0: Higher odds of group membership
• OR < 1.0: Lower odds of group membership
• OR = 2.0: 2× higher odds

SIGNIFICANCE LEVELS
───────────────────
• *** p < 0.001
• ** p < 0.01
• * p < 0.05
• . p < 0.10

MODEL FIT STATISTICS
────────────────────
• Pseudo R²: Variance explained
• Log-likelihood: Model fit
• AIC/BIC: Model comparison

FEATURE IMPORTANCE
──────────────────
Identifies which terms distinguish groups:
• Significant positive: Characteristic of group
• Significant negative: Absent from group

USE CASES
─────────
• Identify distinguishing features
• Predict group membership
• Compare group characteristics
"""
        text_widget = tk.Text(
            parent, font=FONTS.get_font("body"),
            bg=self.theme["bg_card"], fg=self.theme["text_primary"],
            wrap=tk.WORD, padx=16, pady=16, relief=tk.FLAT,
        )
        text_widget.insert("1.0", info_text)
        text_widget.config(state=tk.DISABLED)
        text_widget.pack(fill=tk.BOTH, expand=True)
        
        # Add right-click context menu for copy
        def show_context_menu(event):
            menu = tk.Menu(text_widget, tearoff=0)
            menu.add_command(label="Copy Selected", command=lambda: copy_selected(text_widget))
            menu.add_command(label="Copy All", command=lambda: copy_all(text_widget))
            menu.tk_popup(event.x_root, event.y_root)
        
        def copy_selected(widget):
            try:
                widget.config(state=tk.NORMAL)
                selected = widget.get(tk.SEL_FIRST, tk.SEL_LAST)
                widget.config(state=tk.DISABLED)
                widget.clipboard_clear()
                widget.clipboard_append(selected)
            except tk.TclError:
                pass  # No selection
        
        def copy_all(widget):
            widget.config(state=tk.NORMAL)
            content = widget.get("1.0", tk.END)
            widget.config(state=tk.DISABLED)
            widget.clipboard_clear()
            widget.clipboard_append(content.strip())
        
        text_widget.bind("<Button-3>", show_context_menu)  # Right-click
        text_widget.bind("<Control-c>", lambda e: copy_selected(text_widget))

    def destroy(self):
        """Clean up resources."""
        try:
            plt.close('all')
        except:
            pass
        super().destroy()



# -*- coding: utf-8 -*-
"""
Crosstabs / Contingency Table Analysis
======================================
Analyze associations between categorical variables.

Features:
- Cross-tabulation (frequency and percentage tables)
- Chi-squared test of independence
- Fisher's exact test (for 2x2 tables)
- Cramér's V effect size
- Phi coefficient (for 2x2 tables)
- Expected frequencies
- Standardized residuals
"""

from dataclasses import dataclass, field
from typing import Dict, List, Optional, Tuple, Any

import numpy as np
import pandas as pd


@dataclass
class ChiSquaredResult:
    """Result of Chi-squared test."""
    statistic: float = 0.0
    p_value: float = 0.0
    df: int = 0
    is_significant: bool = False
    min_expected: float = 0.0  # Minimum expected frequency
    cells_below_5: int = 0  # Number of cells with expected < 5
    valid: bool = True  # Whether assumptions are met
    warning: str = ""


@dataclass
class FisherResult:
    """Result of Fisher's exact test."""
    odds_ratio: float = 0.0
    p_value: float = 0.0
    is_significant: bool = False
    ci_lower: float = 0.0
    ci_upper: float = 0.0


@dataclass
class EffectSize:
    """Effect size measures for contingency tables."""
    cramers_v: float = 0.0
    cramers_v_interpretation: str = ""
    phi: float = 0.0  # Only for 2x2 tables
    phi_interpretation: str = ""
    contingency_coef: float = 0.0  # Pearson's contingency coefficient


@dataclass
class CrosstabResult:
    """Complete result of crosstab analysis."""
    # Variables
    row_var: str = ""
    col_var: str = ""
    
    # Dimensions
    n_rows: int = 0
    n_cols: int = 0
    n_total: int = 0
    is_2x2: bool = False
    
    # Tables
    observed: pd.DataFrame = None  # Observed frequencies
    expected: pd.DataFrame = None  # Expected frequencies
    row_pct: pd.DataFrame = None  # Row percentages
    col_pct: pd.DataFrame = None  # Column percentages
    total_pct: pd.DataFrame = None  # Total percentages
    residuals: pd.DataFrame = None  # Standardized residuals
    
    # Tests
    chi_squared: ChiSquaredResult = None
    fisher: FisherResult = None  # Only for 2x2
    
    # Effect sizes
    effect_size: EffectSize = None
    
    # Interpretation
    interpretation: str = ""


def compute_crosstab(
    df: pd.DataFrame,
    row_var: str,
    col_var: str,
    alpha: float = 0.05,
    verbose: bool = True,
) -> CrosstabResult:
    """
    Compute cross-tabulation and statistical tests.
    
    Parameters
    ----------
    df : pd.DataFrame
        Dataset
    row_var : str
        Row variable name (categorical)
    col_var : str
        Column variable name (categorical)
    alpha : float
        Significance level
    verbose : bool
        Print progress
        
    Returns
    -------
    CrosstabResult
    """
    from scipy import stats
    
    result = CrosstabResult(row_var=row_var, col_var=col_var)
    
    # Validate columns
    if row_var not in df.columns:
        raise ValueError(f"Row variable '{row_var}' not found")
    if col_var not in df.columns:
        raise ValueError(f"Column variable '{col_var}' not found")
    
    # Clean data
    clean_df = df[[row_var, col_var]].dropna()
    result.n_total = len(clean_df)
    
    if result.n_total < 2:
        raise ValueError("Insufficient data for analysis")
    
    # Create observed frequency table
    observed = pd.crosstab(clean_df[row_var], clean_df[col_var], margins=True, margins_name="Total")
    
    # Store without margins for calculations
    obs_no_margins = pd.crosstab(clean_df[row_var], clean_df[col_var])
    
    result.observed = observed
    result.n_rows = len(obs_no_margins.index)
    result.n_cols = len(obs_no_margins.columns)
    result.is_2x2 = (result.n_rows == 2 and result.n_cols == 2)
    
    if verbose:
        print(f"Crosstab Analysis: {row_var} × {col_var}")
        print(f"  Dimensions: {result.n_rows} × {result.n_cols}")
        print(f"  Total N: {result.n_total}")
    
    # Calculate percentages
    result.row_pct = pd.crosstab(clean_df[row_var], clean_df[col_var], normalize='index', margins=True, margins_name="Total") * 100
    result.col_pct = pd.crosstab(clean_df[row_var], clean_df[col_var], normalize='columns', margins=True, margins_name="Total") * 100
    result.total_pct = pd.crosstab(clean_df[row_var], clean_df[col_var], normalize='all', margins=True, margins_name="Total") * 100
    
    # Chi-squared test
    chi2_result = ChiSquaredResult()
    try:
        chi2, p, dof, expected = stats.chi2_contingency(obs_no_margins)
        
        chi2_result.statistic = chi2
        chi2_result.p_value = p
        chi2_result.df = dof
        chi2_result.is_significant = p < alpha
        
        # Create expected frequencies DataFrame
        result.expected = pd.DataFrame(
            expected,
            index=obs_no_margins.index,
            columns=obs_no_margins.columns
        )
        
        # Check assumptions
        chi2_result.min_expected = expected.min()
        chi2_result.cells_below_5 = np.sum(expected < 5)
        total_cells = expected.size
        
        if chi2_result.min_expected < 1:
            chi2_result.valid = False
            chi2_result.warning = "Some expected frequencies < 1. Chi-squared may be invalid."
        elif chi2_result.cells_below_5 > 0.2 * total_cells:
            chi2_result.valid = False
            chi2_result.warning = f"More than 20% of cells have expected frequency < 5 ({chi2_result.cells_below_5}/{total_cells} cells)."
        
        # Calculate standardized residuals
        obs_array = obs_no_margins.values
        residuals = (obs_array - expected) / np.sqrt(expected)
        result.residuals = pd.DataFrame(
            residuals,
            index=obs_no_margins.index,
            columns=obs_no_margins.columns
        )
        
    except Exception as e:
        chi2_result.valid = False
        chi2_result.warning = str(e)
    
    result.chi_squared = chi2_result
    
    # Fisher's exact test (for 2x2 tables)
    if result.is_2x2:
        fisher_result = FisherResult()
        try:
            odds_ratio, p_fisher = stats.fisher_exact(obs_no_margins.values)
            fisher_result.odds_ratio = odds_ratio
            fisher_result.p_value = p_fisher
            fisher_result.is_significant = p_fisher < alpha
            
            # Confidence interval for odds ratio (approximate)
            if odds_ratio > 0 and not np.isinf(odds_ratio):
                a, b, c, d = obs_no_margins.values.flatten()
                se_log_or = np.sqrt(1/a + 1/b + 1/c + 1/d) if min(a,b,c,d) > 0 else np.nan
                if not np.isnan(se_log_or):
                    log_or = np.log(odds_ratio)
                    fisher_result.ci_lower = np.exp(log_or - 1.96 * se_log_or)
                    fisher_result.ci_upper = np.exp(log_or + 1.96 * se_log_or)
                    
        except Exception as e:
            if verbose:
                print(f"  Fisher's test error: {e}")
        
        result.fisher = fisher_result
    
    # Effect sizes
    effect = EffectSize()
    
    # Cramér's V
    if chi2_result.statistic > 0:
        min_dim = min(result.n_rows - 1, result.n_cols - 1)
        if min_dim > 0:
            effect.cramers_v = np.sqrt(chi2_result.statistic / (result.n_total * min_dim))
            
            # Interpretation (Cohen's guidelines adapted for Cramér's V)
            if min_dim == 1:  # 2xk or kx2 table
                if effect.cramers_v < 0.1:
                    effect.cramers_v_interpretation = "Negligible"
                elif effect.cramers_v < 0.3:
                    effect.cramers_v_interpretation = "Small"
                elif effect.cramers_v < 0.5:
                    effect.cramers_v_interpretation = "Medium"
                else:
                    effect.cramers_v_interpretation = "Large"
            else:  # Larger tables
                if effect.cramers_v < 0.07:
                    effect.cramers_v_interpretation = "Negligible"
                elif effect.cramers_v < 0.21:
                    effect.cramers_v_interpretation = "Small"
                elif effect.cramers_v < 0.35:
                    effect.cramers_v_interpretation = "Medium"
                else:
                    effect.cramers_v_interpretation = "Large"
    
    # Phi coefficient (only meaningful for 2x2)
    if result.is_2x2:
        effect.phi = np.sqrt(chi2_result.statistic / result.n_total)
        if effect.phi < 0.1:
            effect.phi_interpretation = "Negligible"
        elif effect.phi < 0.3:
            effect.phi_interpretation = "Small"
        elif effect.phi < 0.5:
            effect.phi_interpretation = "Medium"
        else:
            effect.phi_interpretation = "Large"
    
    # Contingency coefficient
    if chi2_result.statistic > 0:
        effect.contingency_coef = np.sqrt(chi2_result.statistic / (chi2_result.statistic + result.n_total))
    
    result.effect_size = effect
    
    # Generate interpretation
    if chi2_result.is_significant:
        result.interpretation = (
            f"There is a statistically significant association between {row_var} and {col_var} "
            f"(χ²({chi2_result.df}) = {chi2_result.statistic:.3f}, p = {chi2_result.p_value:.4f}). "
        )
        if result.is_2x2:
            result.interpretation += f"Effect size: φ = {effect.phi:.3f} ({effect.phi_interpretation}). "
            if result.fisher:
                result.interpretation += f"Odds ratio = {result.fisher.odds_ratio:.2f}."
        else:
            result.interpretation += f"Effect size: Cramér's V = {effect.cramers_v:.3f} ({effect.cramers_v_interpretation})."
    else:
        result.interpretation = (
            f"No statistically significant association was found between {row_var} and {col_var} "
            f"(χ²({chi2_result.df}) = {chi2_result.statistic:.3f}, p = {chi2_result.p_value:.4f})."
        )
    
    if chi2_result.warning:
        result.interpretation += f"\n\nWarning: {chi2_result.warning}"
        if result.is_2x2 and result.fisher:
            result.interpretation += f" Consider using Fisher's exact test (p = {result.fisher.p_value:.4f})."
    
    if verbose:
        print(f"  Chi-squared: χ²({chi2_result.df}) = {chi2_result.statistic:.3f}, p = {chi2_result.p_value:.4f}")
        if result.is_2x2 and result.fisher:
            print(f"  Fisher's exact: p = {result.fisher.p_value:.4f}, OR = {result.fisher.odds_ratio:.3f}")
        print(f"  Cramér's V: {effect.cramers_v:.3f} ({effect.cramers_v_interpretation})")
    
    return result


def get_categorical_columns(df: pd.DataFrame, max_categories: int = 50) -> List[str]:
    """Get list of columns suitable for crosstab analysis."""
    cat_cols = []
    for col in df.columns:
        n_unique = df[col].nunique()
        if 2 <= n_unique <= max_categories:
            cat_cols.append(col)
    return cat_cols


def format_crosstab_table(
    observed: pd.DataFrame,
    row_pct: pd.DataFrame = None,
    col_pct: pd.DataFrame = None,
    show_pct: str = "row",  # "row", "col", "both", "none"
) -> pd.DataFrame:
    """
    Format crosstab with counts and percentages.
    
    Parameters
    ----------
    observed : pd.DataFrame
        Observed frequencies
    row_pct : pd.DataFrame
        Row percentages
    col_pct : pd.DataFrame
        Column percentages
    show_pct : str
        Which percentages to show
        
    Returns
    -------
    pd.DataFrame
        Formatted table with counts and percentages
    """
    if show_pct == "none" or (row_pct is None and col_pct is None):
        return observed.copy()
    
    result = observed.copy().astype(str)
    
    for row in observed.index:
        for col in observed.columns:
            count = observed.loc[row, col]
            
            if show_pct == "row" and row_pct is not None:
                pct = row_pct.loc[row, col]
                result.loc[row, col] = f"{count} ({pct:.1f}%)"
            elif show_pct == "col" and col_pct is not None:
                pct = col_pct.loc[row, col]
                result.loc[row, col] = f"{count} ({pct:.1f}%)"
            elif show_pct == "both":
                r_pct = row_pct.loc[row, col] if row_pct is not None else 0
                c_pct = col_pct.loc[row, col] if col_pct is not None else 0
                result.loc[row, col] = f"{count}\n({r_pct:.1f}% | {c_pct:.1f}%)"
    
    return result



# -*- coding: utf-8 -*-
"""
Crosstabs Panel - Contingency Table Analysis
"""

import tkinter as tk
from tkinter import ttk, messagebox, filedialog

from biblium.gui.config import FONTS, get_theme
from biblium.gui.core.events import event_bus, EventBus
from biblium.gui.panels.base import BasePanel
from biblium.gui.widgets.buttons import ActionButton

try:
    import pandas as pd
    import numpy as np
    HAS_PANDAS = True
except ImportError:
    HAS_PANDAS = False


class CrosstabsPanel(BasePanel):
    """Panel for contingency table analysis - SPSS-like interface."""
    
    title = "Crosstabs"
    icon = "📋"
    requires_data = True
    
    def __init__(self, parent, theme="light", **kwargs):
        self._result = None
        super().__init__(parent, theme=theme, **kwargs)
        event_bus.subscribe(EventBus.DATASET_LOADED, self._handle_data_loaded)
    
    def _handle_data_loaded(self, data):
        bib = data.get("bib") if isinstance(data, dict) else data
        self.on_data_loaded(bib)
    
    def _create_options(self):
        self._add_title()
        
        # Variable selection frame
        var_frame = tk.Frame(self.options_content, bg=self.theme["bg_secondary"])
        var_frame.pack(fill=tk.BOTH, expand=True, padx=8, pady=4)
        
        # Left: Available variables
        left_frame = tk.Frame(var_frame, bg=self.theme["bg_secondary"])
        left_frame.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)
        
        tk.Label(left_frame, text="Available Variables:", font=FONTS.get_font("body", bold=True),
                bg=self.theme["bg_secondary"], fg=self.theme["text_primary"]).pack(anchor="w")
        
        avail_frame = tk.Frame(left_frame)
        avail_frame.pack(fill=tk.BOTH, expand=True, pady=4)
        
        self.avail_listbox = tk.Listbox(avail_frame, height=10, width=20, font=FONTS.get_font("small"),
                                        selectmode=tk.EXTENDED, exportselection=False, bg="white")
        self.avail_listbox.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)
        avail_sb = ttk.Scrollbar(avail_frame, command=self.avail_listbox.yview)
        avail_sb.pack(side=tk.RIGHT, fill=tk.Y)
        self.avail_listbox.config(yscrollcommand=avail_sb.set)
        
        # Middle: Arrow buttons
        mid_frame = tk.Frame(var_frame, bg=self.theme["bg_secondary"])
        mid_frame.pack(side=tk.LEFT, padx=6, pady=10)
        
        tk.Label(mid_frame, text="Row:", font=FONTS.get_font("small"),
                bg=self.theme["bg_secondary"], fg=self.theme["text_muted"]).pack()
        tk.Button(mid_frame, text="▶", width=3, command=self._add_row).pack(pady=2)
        tk.Button(mid_frame, text="◀", width=3, command=self._remove_row).pack(pady=2)
        
        tk.Label(mid_frame, text="Column:", font=FONTS.get_font("small"),
                bg=self.theme["bg_secondary"], fg=self.theme["text_muted"]).pack(pady=(12,0))
        tk.Button(mid_frame, text="▶", width=3, command=self._add_col).pack(pady=2)
        tk.Button(mid_frame, text="◀", width=3, command=self._remove_col).pack(pady=2)
        
        # Right: Selected variables
        right_frame = tk.Frame(var_frame, bg=self.theme["bg_secondary"])
        right_frame.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)
        
        tk.Label(right_frame, text="Row Variable:", font=FONTS.get_font("body", bold=True),
                bg=self.theme["bg_secondary"], fg=self.theme["text_primary"]).pack(anchor="w")
        self.row_listbox = tk.Listbox(right_frame, height=2, width=20, font=FONTS.get_font("small"),
                                      selectmode=tk.SINGLE, exportselection=False, bg="white")
        self.row_listbox.pack(fill=tk.X, pady=4)
        
        self.row_info_label = tk.Label(right_frame, text="", font=FONTS.get_font("small"),
                                       bg=self.theme["bg_secondary"], fg=self.theme["text_muted"])
        self.row_info_label.pack(anchor="w")
        
        tk.Label(right_frame, text="Column Variable:", font=FONTS.get_font("body", bold=True),
                bg=self.theme["bg_secondary"], fg=self.theme["text_primary"]).pack(anchor="w", pady=(8,0))
        self.col_listbox = tk.Listbox(right_frame, height=2, width=20, font=FONTS.get_font("small"),
                                      selectmode=tk.SINGLE, exportselection=False, bg="white")
        self.col_listbox.pack(fill=tk.X, pady=4)
        
        self.col_info_label = tk.Label(right_frame, text="", font=FONTS.get_font("small"),
                                       bg=self.theme["bg_secondary"], fg=self.theme["text_muted"])
        self.col_info_label.pack(anchor="w")
        
        # Options frame
        opt_frame = tk.LabelFrame(self.options_content, text="Options", font=FONTS.get_font("body"),
                                  bg=self.theme["bg_secondary"], fg=self.theme["text_primary"])
        opt_frame.pack(fill=tk.X, padx=8, pady=8)
        
        # Statistics checkboxes
        self.chi_squared_var = tk.BooleanVar(value=True)
        tk.Checkbutton(opt_frame, text="Chi-squared test", variable=self.chi_squared_var,
                      font=FONTS.get_font("body"), bg=self.theme["bg_secondary"],
                      selectcolor="white").pack(anchor="w", padx=8, pady=2)
        
        self.fisher_var = tk.BooleanVar(value=True)
        tk.Checkbutton(opt_frame, text="Fisher's exact test (2×2)", variable=self.fisher_var,
                      font=FONTS.get_font("body"), bg=self.theme["bg_secondary"],
                      selectcolor="white").pack(anchor="w", padx=8, pady=2)
        
        self.effect_size_var = tk.BooleanVar(value=True)
        tk.Checkbutton(opt_frame, text="Effect size (Cramér's V, Phi)", variable=self.effect_size_var,
                      font=FONTS.get_font("body"), bg=self.theme["bg_secondary"],
                      selectcolor="white").pack(anchor="w", padx=8, pady=2)
        
        self.expected_var = tk.BooleanVar(value=True)
        tk.Checkbutton(opt_frame, text="Expected frequencies", variable=self.expected_var,
                      font=FONTS.get_font("body"), bg=self.theme["bg_secondary"],
                      selectcolor="white").pack(anchor="w", padx=8, pady=2)
        
        self.residuals_var = tk.BooleanVar(value=False)
        tk.Checkbutton(opt_frame, text="Standardized residuals", variable=self.residuals_var,
                      font=FONTS.get_font("body"), bg=self.theme["bg_secondary"],
                      selectcolor="white").pack(anchor="w", padx=8, pady=2)
        
        # Percentages
        pct_frame = tk.Frame(opt_frame, bg=self.theme["bg_secondary"])
        pct_frame.pack(fill=tk.X, padx=8, pady=4)
        tk.Label(pct_frame, text="Show percentages:", font=FONTS.get_font("body"),
                bg=self.theme["bg_secondary"], fg=self.theme["text_primary"]).pack(side=tk.LEFT)
        self.pct_var = tk.StringVar(value="Row")
        ttk.Combobox(pct_frame, textvariable=self.pct_var, 
                    values=["None", "Row", "Column", "Total"],
                    state="readonly", width=10).pack(side=tk.LEFT, padx=8)
        
        # Alpha
        alpha_frame = tk.Frame(opt_frame, bg=self.theme["bg_secondary"])
        alpha_frame.pack(fill=tk.X, padx=8, pady=4)
        tk.Label(alpha_frame, text="Alpha:", font=FONTS.get_font("body"),
                bg=self.theme["bg_secondary"], fg=self.theme["text_primary"]).pack(side=tk.LEFT)
        self.alpha_var = tk.StringVar(value="0.05")
        ttk.Combobox(alpha_frame, textvariable=self.alpha_var, values=["0.01","0.05","0.10"],
                    state="readonly", width=6).pack(side=tk.LEFT, padx=8)
        
        # Buttons
        # LLM Configuration (v2.11)
        self._create_llm_config_card()
        
        btn_frame = tk.Frame(self.options_content, bg=self.theme["bg_secondary"])
        btn_frame.pack(fill=tk.X, padx=8, pady=8)
        
        ActionButton(btn_frame, text="OK - Run Analysis", icon="▶", command=self._run_analysis,
                    theme=self.theme_name).pack(fill=tk.X)
        ActionButton(btn_frame, text="Reset", icon="🔄", command=self._reset_selection,
                    theme=self.theme_name).pack(fill=tk.X, pady=(4,0))
        ActionButton(btn_frame, text="Export Results", icon="📊", command=self._export_results,
                    theme=self.theme_name).pack(fill=tk.X, pady=(4,0))
        
        # Populate if data already loaded
        if hasattr(self, 'bib') and self.bib is not None and hasattr(self.bib, 'df'):
            self._populate_variables()
    
    def _add_row(self):
        selection = self.avail_listbox.curselection()
        if not selection:
            return
        if self.row_listbox.size() > 0:
            messagebox.showinfo("Info", "Only one row variable allowed. Remove current first.")
            return
        idx = selection[0]
        var = self.avail_listbox.get(idx)
        self.row_listbox.insert(tk.END, var)
        self.avail_listbox.delete(idx)
        self._update_var_info()
    
    def _remove_row(self):
        if self.row_listbox.size() > 0:
            var = self.row_listbox.get(0)
            self.avail_listbox.insert(tk.END, var)
            self.row_listbox.delete(0)
            self.row_info_label.config(text="")
    
    def _add_col(self):
        selection = self.avail_listbox.curselection()
        if not selection:
            return
        if self.col_listbox.size() > 0:
            messagebox.showinfo("Info", "Only one column variable allowed. Remove current first.")
            return
        idx = selection[0]
        var = self.avail_listbox.get(idx)
        self.col_listbox.insert(tk.END, var)
        self.avail_listbox.delete(idx)
        self._update_var_info()
    
    def _remove_col(self):
        if self.col_listbox.size() > 0:
            var = self.col_listbox.get(0)
            self.avail_listbox.insert(tk.END, var)
            self.col_listbox.delete(0)
            self.col_info_label.config(text="")
    
    def _update_var_info(self):
        if self.row_listbox.size() > 0 and self.bib:
            var = self.row_listbox.get(0)
            n = self.bib.df[var].nunique()
            vals = self.bib.df[var].value_counts()
            info = f"Categories: {n} | " + ", ".join([f"{k}" for k in vals.head(4).index])
            if n > 4:
                info += "..."
            self.row_info_label.config(text=info)
        
        if self.col_listbox.size() > 0 and self.bib:
            var = self.col_listbox.get(0)
            n = self.bib.df[var].nunique()
            vals = self.bib.df[var].value_counts()
            info = f"Categories: {n} | " + ", ".join([f"{k}" for k in vals.head(4).index])
            if n > 4:
                info += "..."
            self.col_info_label.config(text=info)
    
    def _reset_selection(self):
        self._remove_row()
        self._remove_col()
        self._result = None
        self._show_placeholder()
    
    def _create_results(self):
        self.results_card = tk.Frame(self.results_frame, bg=self.theme["bg_card"],
                                     highlightbackground=self.theme["border"], highlightthickness=1)
        self.results_card.pack(fill=tk.BOTH, expand=True, padx=4, pady=4)
        
        self.notebook = ttk.Notebook(self.results_card)
        self.notebook.pack(fill=tk.BOTH, expand=True, padx=4, pady=4)
        
        self.crosstab_frame = tk.Frame(self.notebook, bg=self.theme["bg_card"])
        self.notebook.add(self.crosstab_frame, text="📋 Crosstab")
        
        self.stats_frame = tk.Frame(self.notebook, bg=self.theme["bg_card"])
        self.notebook.add(self.stats_frame, text="🧪 Statistics")
        
        self.expected_frame = tk.Frame(self.notebook, bg=self.theme["bg_card"])
        self.notebook.add(self.expected_frame, text="📊 Expected")
        
        self.summary_frame = tk.Frame(self.notebook, bg=self.theme["bg_card"])
        self.notebook.add(self.summary_frame, text="📝 Summary")

        

        # Info tab

        info_frame = tk.Frame(self.notebook, bg=self.theme["bg_card"])

        self.notebook.add(info_frame, text="ℹ️ Info")

        self._create_info_content(info_frame)
        
        self._show_placeholder()
    
    def _show_placeholder(self):
        for f in [self.crosstab_frame, self.stats_frame, self.expected_frame, self.summary_frame]:
            for w in f.winfo_children():
                w.destroy()
        tk.Label(self.crosstab_frame, 
                text="Select Row and Column variables\nthen click 'OK - Run Analysis'\n\nAnalyzes association between two categorical variables.",
                font=FONTS.get_font("body"), bg=self.theme["bg_card"], fg=self.theme["text_muted"],
                justify=tk.CENTER).pack(expand=True)
    
    def _run_analysis(self):
        if not self.bib:
            messagebox.showwarning("No Data", "Load a dataset first.")
            return
        if self.row_listbox.size() == 0:
            messagebox.showwarning("Required", "Select a row variable.")
            return
        if self.col_listbox.size() == 0:
            messagebox.showwarning("Required", "Select a column variable.")
            return
        
        row_var = self.row_listbox.get(0)
        col_var = self.col_listbox.get(0)
        alpha = float(self.alpha_var.get())
        
        try:
            from biblium.crosstabs import compute_crosstab
            
            result = compute_crosstab(self.bib.df, row_var, col_var, alpha=alpha, verbose=True)
            self._result = result
            
            self._update_crosstab()
            self._update_statistics()
            self._update_expected()
            self._update_summary()
            
        except Exception as e:
            import traceback
            traceback.print_exc()
            messagebox.showerror("Error", f"Analysis failed:\n{e}")
    
    def _update_crosstab(self):
        for w in self.crosstab_frame.winfo_children():
            w.destroy()
        if not self._result:
            return
        
        r = self._result
        
        tk.Label(self.crosstab_frame, text=f"Crosstab: {r.row_var} × {r.col_var}",
                font=FONTS.get_font("heading", bold=True), bg=self.theme["bg_card"],
                fg=self.theme["text_primary"]).pack(pady=(8,4), anchor="w", padx=8)
        
        tk.Label(self.crosstab_frame, text=f"N = {r.n_total}  |  {r.n_rows} × {r.n_cols} table",
                font=FONTS.get_font("body"), bg=self.theme["bg_card"],
                fg=self.theme["text_secondary"]).pack(anchor="w", padx=8)
        
        # Create table
        tree_frame = tk.Frame(self.crosstab_frame, bg=self.theme["bg_card"])
        tree_frame.pack(fill=tk.BOTH, expand=True, padx=8, pady=8)
        
        observed = r.observed
        cols = [r.row_var] + list(observed.columns)
        tree = ttk.Treeview(tree_frame, columns=cols, show="headings", height=min(15, len(observed)+1))
        
        for c in cols:
            tree.heading(c, text=str(c))
            tree.column(c, width=100, minwidth=80, anchor="center")
        tree.column(cols[0], anchor="w", width=120)
        
        # Get percentages based on selection
        pct_type = self.pct_var.get().lower()
        
        for idx in observed.index:
            values = [str(idx)]
            for col in observed.columns:
                count = observed.loc[idx, col]
                pct_value = None
                
                # Try to get percentage, handle missing indices gracefully
                try:
                    if pct_type == "row" and r.row_pct is not None and idx in r.row_pct.index and col in r.row_pct.columns:
                        pct_value = r.row_pct.loc[idx, col]
                    elif pct_type == "column" and r.col_pct is not None and idx in r.col_pct.index and col in r.col_pct.columns:
                        pct_value = r.col_pct.loc[idx, col]
                    elif pct_type == "total" and r.total_pct is not None and idx in r.total_pct.index and col in r.total_pct.columns:
                        pct_value = r.total_pct.loc[idx, col]
                except (KeyError, TypeError):
                    pct_value = None
                
                if pct_value is not None:
                    values.append(f"{count} ({pct_value:.1f}%)")
                else:
                    values.append(str(count))
            
            tag = 'total' if idx == 'Total' else ''
            tree.insert("", tk.END, values=values, tags=(tag,))
        
        tree.tag_configure('total', background='#e8f4f8')
        
        vsb = ttk.Scrollbar(tree_frame, orient="vertical", command=tree.yview)
        hsb = ttk.Scrollbar(tree_frame, orient="horizontal", command=tree.xview)
        tree.configure(yscrollcommand=vsb.set, xscrollcommand=hsb.set)
        tree.grid(row=0, column=0, sticky="nsew")
        vsb.grid(row=0, column=1, sticky="ns")
        hsb.grid(row=1, column=0, sticky="ew")
        tree_frame.grid_rowconfigure(0, weight=1)
        tree_frame.grid_columnconfigure(0, weight=1)
    
    def _update_statistics(self):
        for w in self.stats_frame.winfo_children():
            w.destroy()
        if not self._result:
            return
        
        r = self._result
        
        canvas = tk.Canvas(self.stats_frame, bg=self.theme["bg_card"], highlightthickness=0)
        sb = ttk.Scrollbar(self.stats_frame, orient="vertical", command=canvas.yview)
        inner = tk.Frame(canvas, bg=self.theme["bg_card"])
        inner.bind("<Configure>", lambda e: canvas.configure(scrollregion=canvas.bbox("all")))
        canvas.create_window((0,0), window=inner, anchor="nw")
        canvas.configure(yscrollcommand=sb.set)
        canvas.pack(side="left", fill="both", expand=True)
        sb.pack(side="right", fill="y")
        
        tk.Label(inner, text="Statistical Tests", font=FONTS.get_font("heading", bold=True),
                bg=self.theme["bg_card"], fg=self.theme["text_primary"]).pack(pady=(8,4), anchor="w", padx=8)
        
        # Chi-squared test
        if self.chi_squared_var.get():
            chi_frame = tk.LabelFrame(inner, text="Chi-Squared Test", font=FONTS.get_font("body"),
                                      bg=self.theme["bg_card"], fg=self.theme["text_primary"])
            chi_frame.pack(fill=tk.X, padx=8, pady=4)
            
            c = r.chi_squared
            chi_text = f"χ²({c.df}) = {c.statistic:.4f}\np-value = {c.p_value:.4f}\n"
            chi_text += f"Result: {'Significant *' if c.is_significant else 'Not significant'}"
            
            chi_color = "green" if c.is_significant else self.theme["text_primary"]
            tk.Label(chi_frame, text=chi_text, font=FONTS.get_font("code"),
                    bg=self.theme["bg_card"], fg=chi_color, justify=tk.LEFT).pack(padx=8, pady=4, anchor="w")
            
            if c.warning:
                tk.Label(chi_frame, text=f"⚠️ {c.warning}", font=FONTS.get_font("small"),
                        bg=self.theme["bg_card"], fg="orange", wraplength=400,
                        justify=tk.LEFT).pack(padx=8, pady=4, anchor="w")
        
        # Fisher's exact test (2x2 only)
        if self.fisher_var.get() and r.is_2x2 and r.fisher:
            fish_frame = tk.LabelFrame(inner, text="Fisher's Exact Test (2×2)", font=FONTS.get_font("body"),
                                       bg=self.theme["bg_card"], fg=self.theme["text_primary"])
            fish_frame.pack(fill=tk.X, padx=8, pady=4)
            
            f = r.fisher
            fish_text = f"p-value = {f.p_value:.4f}\n"
            fish_text += f"Odds Ratio = {f.odds_ratio:.3f}"
            if f.ci_lower > 0 and f.ci_upper > 0:
                fish_text += f"\n95% CI: [{f.ci_lower:.3f}, {f.ci_upper:.3f}]"
            fish_text += f"\nResult: {'Significant *' if f.is_significant else 'Not significant'}"
            
            fish_color = "green" if f.is_significant else self.theme["text_primary"]
            tk.Label(fish_frame, text=fish_text, font=FONTS.get_font("code"),
                    bg=self.theme["bg_card"], fg=fish_color, justify=tk.LEFT).pack(padx=8, pady=4, anchor="w")
        
        # Effect sizes
        if self.effect_size_var.get():
            eff_frame = tk.LabelFrame(inner, text="Effect Size", font=FONTS.get_font("body"),
                                      bg=self.theme["bg_card"], fg=self.theme["text_primary"])
            eff_frame.pack(fill=tk.X, padx=8, pady=4)
            
            e = r.effect_size
            eff_text = f"Cramér's V = {e.cramers_v:.4f} ({e.cramers_v_interpretation})\n"
            if r.is_2x2:
                eff_text += f"Phi (φ) = {e.phi:.4f} ({e.phi_interpretation})\n"
            eff_text += f"Contingency Coefficient = {e.contingency_coef:.4f}"
            
            tk.Label(eff_frame, text=eff_text, font=FONTS.get_font("code"),
                    bg=self.theme["bg_card"], fg=self.theme["text_primary"],
                    justify=tk.LEFT).pack(padx=8, pady=4, anchor="w")
    
    def _update_expected(self):
        for w in self.expected_frame.winfo_children():
            w.destroy()
        if not self._result:
            return
        
        r = self._result
        
        canvas = tk.Canvas(self.expected_frame, bg=self.theme["bg_card"], highlightthickness=0)
        sb = ttk.Scrollbar(self.expected_frame, orient="vertical", command=canvas.yview)
        inner = tk.Frame(canvas, bg=self.theme["bg_card"])
        inner.bind("<Configure>", lambda e: canvas.configure(scrollregion=canvas.bbox("all")))
        canvas.create_window((0,0), window=inner, anchor="nw")
        canvas.configure(yscrollcommand=sb.set)
        canvas.pack(side="left", fill="both", expand=True)
        sb.pack(side="right", fill="y")
        
        # Expected frequencies
        if self.expected_var.get() and r.expected is not None:
            tk.Label(inner, text="Expected Frequencies", font=FONTS.get_font("heading", bold=True),
                    bg=self.theme["bg_card"], fg=self.theme["text_primary"]).pack(pady=(8,4), anchor="w", padx=8)
            
            exp_frame = tk.Frame(inner, bg=self.theme["bg_card"])
            exp_frame.pack(fill=tk.X, padx=8, pady=4)
            
            expected = r.expected
            cols = [r.row_var] + list(expected.columns)
            tree = ttk.Treeview(exp_frame, columns=cols, show="headings", height=min(10, len(expected)+1))
            
            for c in cols:
                tree.heading(c, text=str(c))
                tree.column(c, width=100, anchor="center")
            tree.column(cols[0], anchor="w", width=120)
            
            for idx in expected.index:
                values = [str(idx)] + [f"{expected.loc[idx, col]:.2f}" for col in expected.columns]
                tree.insert("", tk.END, values=values)
            
            tree.pack(fill=tk.X)
            
            # Check for cells < 5
            c = r.chi_squared
            tk.Label(inner, text=f"Min expected: {c.min_expected:.2f}  |  Cells < 5: {c.cells_below_5}",
                    font=FONTS.get_font("small"), bg=self.theme["bg_card"],
                    fg="orange" if c.cells_below_5 > 0 else self.theme["text_secondary"]).pack(anchor="w", padx=8, pady=4)
        
        # Standardized residuals
        if self.residuals_var.get() and r.residuals is not None:
            tk.Label(inner, text="Standardized Residuals", font=FONTS.get_font("heading", bold=True),
                    bg=self.theme["bg_card"], fg=self.theme["text_primary"]).pack(pady=(16,4), anchor="w", padx=8)
            
            tk.Label(inner, text="Values > |1.96| indicate significant deviation from expected",
                    font=FONTS.get_font("small"), bg=self.theme["bg_card"],
                    fg=self.theme["text_muted"]).pack(anchor="w", padx=8)
            
            res_frame = tk.Frame(inner, bg=self.theme["bg_card"])
            res_frame.pack(fill=tk.X, padx=8, pady=4)
            
            residuals = r.residuals
            cols = [r.row_var] + list(residuals.columns)
            tree = ttk.Treeview(res_frame, columns=cols, show="headings", height=min(10, len(residuals)+1))
            
            for c in cols:
                tree.heading(c, text=str(c))
                tree.column(c, width=100, anchor="center")
            tree.column(cols[0], anchor="w", width=120)
            
            for idx in residuals.index:
                values = [str(idx)] + [f"{residuals.loc[idx, col]:.3f}" for col in residuals.columns]
                tree.insert("", tk.END, values=values)
            
            tree.pack(fill=tk.X)
    
    def _update_summary(self):
        for w in self.summary_frame.winfo_children():
            w.destroy()
        if not self._result:
            return
        
        r = self._result
        
        txt = "CROSSTABS ANALYSIS REPORT\n" + "="*50 + "\n\n"
        txt += f"Row Variable: {r.row_var}\n"
        txt += f"Column Variable: {r.col_var}\n"
        txt += f"Table Size: {r.n_rows} × {r.n_cols}\n"
        txt += f"Total N: {r.n_total}\n\n"
        
        txt += "OBSERVED FREQUENCIES\n" + "-"*30 + "\n"
        txt += r.observed.to_string() + "\n\n"
        
        txt += "STATISTICAL TESTS\n" + "-"*30 + "\n"
        c = r.chi_squared
        txt += f"Chi-Squared: χ²({c.df}) = {c.statistic:.4f}, p = {c.p_value:.4f}\n"
        
        if r.is_2x2 and r.fisher:
            f = r.fisher
            txt += f"Fisher's Exact: p = {f.p_value:.4f}, OR = {f.odds_ratio:.3f}\n"
        
        txt += f"\nEFFECT SIZE\n" + "-"*30 + "\n"
        e = r.effect_size
        txt += f"Cramér's V = {e.cramers_v:.4f} ({e.cramers_v_interpretation})\n"
        if r.is_2x2:
            txt += f"Phi = {e.phi:.4f} ({e.phi_interpretation})\n"
        
        txt += f"\nINTERPRETATION\n" + "-"*30 + "\n"
        txt += r.interpretation + "\n"
        
        text_w = tk.Text(self.summary_frame, font=FONTS.get_font("code"), bg=self.theme["bg_card"],
                        fg=self.theme["text_primary"], wrap=tk.WORD)
        text_w.pack(fill=tk.BOTH, expand=True, padx=8, pady=8)
        text_w.insert("1.0", txt)
        text_w.config(state=tk.DISABLED)
    
    def _export_results(self):
        if not self._result:
            messagebox.showwarning("No Results", "Run analysis first.")
            return
        filepath = filedialog.asksaveasfilename(defaultextension=".xlsx",
                                                filetypes=[("Excel","*.xlsx"),("Text","*.txt")])
        if not filepath:
            return
        try:
            r = self._result
            if filepath.endswith('.xlsx'):
                with pd.ExcelWriter(filepath, engine='openpyxl') as writer:
                    r.observed.to_excel(writer, sheet_name='Observed')
                    if r.expected is not None:
                        r.expected.to_excel(writer, sheet_name='Expected')
                    if r.row_pct is not None:
                        r.row_pct.to_excel(writer, sheet_name='Row_Pct')
                    if r.col_pct is not None:
                        r.col_pct.to_excel(writer, sheet_name='Col_Pct')
                    if r.residuals is not None:
                        r.residuals.to_excel(writer, sheet_name='Residuals')
                    
                    # Statistics summary
                    stats_data = {
                        'Test': ['Chi-Squared', 'df', 'p-value', "Cramér's V"],
                        'Value': [r.chi_squared.statistic, r.chi_squared.df, 
                                 r.chi_squared.p_value, r.effect_size.cramers_v]
                    }
                    if r.is_2x2 and r.fisher:
                        stats_data['Test'].extend(['Fisher p-value', 'Odds Ratio', 'Phi'])
                        stats_data['Value'].extend([r.fisher.p_value, r.fisher.odds_ratio, r.effect_size.phi])
                    pd.DataFrame(stats_data).to_excel(writer, sheet_name='Statistics', index=False)
            else:
                with open(filepath, 'w') as f:
                    for w in self.summary_frame.winfo_children():
                        if isinstance(w, tk.Text):
                            f.write(w.get("1.0", tk.END))
            messagebox.showinfo("Exported", f"Saved to:\n{filepath}")
        except Exception as e:
            messagebox.showerror("Error", f"Export failed:\n{e}")
    
    def _populate_variables(self):
        self.avail_listbox.delete(0, tk.END)
        self.row_listbox.delete(0, tk.END)
        self.col_listbox.delete(0, tk.END)
        self.row_info_label.config(text="")
        self.col_info_label.config(text="")
        if self.bib is not None and hasattr(self.bib, 'df'):
            for col in self.bib.df.columns:
                self.avail_listbox.insert(tk.END, col)
    
    def _create_info_content(self, parent):
        """Create Info tab with documentation."""
        info_text = """
CROSSTABULATION
═══════════════

Analyze relationships between categorical variables.

OUTPUT TABLE
────────────
• Observed frequencies
• Expected frequencies (if independent)
• Row percentages
• Column percentages
• Total percentages

STATISTICAL TESTS
─────────────────
• Chi-Square (χ²)
  Tests independence
  H0: Variables are independent
  p < 0.05: Significant association
  
• Fisher's Exact Test
  For 2×2 tables
  Better for small samples
  
• Cramér's V
  Effect size (0 to 1)
  0.1 small, 0.3 medium, 0.5 large

ASSUMPTIONS
───────────
• Expected freq ≥ 5 (most cells)
• Use Fisher for small samples
• Independent observations

USE CASES
─────────
• Document type × Source type
• Country × Collaboration type
• Subject × Open access status
"""
        text_widget = tk.Text(
            parent, font=FONTS.get_font("body"),
            bg=self.theme["bg_card"], fg=self.theme["text_primary"],
            wrap=tk.WORD, padx=16, pady=16, relief=tk.FLAT,
        )
        text_widget.insert("1.0", info_text)
        text_widget.config(state=tk.DISABLED)
        text_widget.pack(fill=tk.BOTH, expand=True)
        
        # Add right-click context menu for copy
        def show_context_menu(event):
            menu = tk.Menu(text_widget, tearoff=0)
            menu.add_command(label="Copy Selected", command=lambda: copy_selected(text_widget))
            menu.add_command(label="Copy All", command=lambda: copy_all(text_widget))
            menu.tk_popup(event.x_root, event.y_root)
        
        def copy_selected(widget):
            try:
                widget.config(state=tk.NORMAL)
                selected = widget.get(tk.SEL_FIRST, tk.SEL_LAST)
                widget.config(state=tk.DISABLED)
                widget.clipboard_clear()
                widget.clipboard_append(selected)
            except tk.TclError:
                pass  # No selection
        
        def copy_all(widget):
            widget.config(state=tk.NORMAL)
            content = widget.get("1.0", tk.END)
            widget.config(state=tk.DISABLED)
            widget.clipboard_clear()
            widget.clipboard_append(content.strip())
        
        text_widget.bind("<Button-3>", show_context_menu)  # Right-click
        text_widget.bind("<Control-c>", lambda e: copy_selected(text_widget))

    def on_data_loaded(self, bib):
        if isinstance(bib, dict):
            bib = bib.get("bib", bib)
        self.bib = bib
        self._result = None
        self._populate_variables()
        self._show_placeholder()
